# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class NetworkMapping(pulumi.CustomResource):
    name: pulumi.Output[str]
    """
    The name of the network mapping.
    """
    recovery_vault_name: pulumi.Output[str]
    """
    The name of the vault that should be updated.
    """
    resource_group_name: pulumi.Output[str]
    """
    Name of the resource group where the vault that should be updated is located.
    """
    source_network_id: pulumi.Output[str]
    """
    The id of the primary network.
    """
    source_recovery_fabric_name: pulumi.Output[str]
    """
    Specifies the ASR fabric where mapping should be created.
    """
    target_network_id: pulumi.Output[str]
    """
    The id of the recovery network.
    """
    target_recovery_fabric_name: pulumi.Output[str]
    """
    The Azure Site Recovery fabric object corresponding to the recovery Azure region.
    """
    def __init__(__self__, resource_name, opts=None, name=None, recovery_vault_name=None, resource_group_name=None, source_network_id=None, source_recovery_fabric_name=None, target_network_id=None, target_recovery_fabric_name=None, __props__=None, __name__=None, __opts__=None):
        """
        Manages a site recovery network mapping on Azure. A network mapping decides how to translate connected netwroks when a VM is migrated from one region to another.
        
        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] name: The name of the network mapping.
        :param pulumi.Input[str] recovery_vault_name: The name of the vault that should be updated.
        :param pulumi.Input[str] resource_group_name: Name of the resource group where the vault that should be updated is located.
        :param pulumi.Input[str] source_network_id: The id of the primary network.
        :param pulumi.Input[str] source_recovery_fabric_name: Specifies the ASR fabric where mapping should be created.
        :param pulumi.Input[str] target_network_id: The id of the recovery network.
        :param pulumi.Input[str] target_recovery_fabric_name: The Azure Site Recovery fabric object corresponding to the recovery Azure region.

        > This content is derived from https://github.com/terraform-providers/terraform-provider-azurerm/blob/master/website/docs/r/recovery_network_mapping.html.markdown.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['name'] = name
            if recovery_vault_name is None:
                raise TypeError("Missing required property 'recovery_vault_name'")
            __props__['recovery_vault_name'] = recovery_vault_name
            if resource_group_name is None:
                raise TypeError("Missing required property 'resource_group_name'")
            __props__['resource_group_name'] = resource_group_name
            if source_network_id is None:
                raise TypeError("Missing required property 'source_network_id'")
            __props__['source_network_id'] = source_network_id
            if source_recovery_fabric_name is None:
                raise TypeError("Missing required property 'source_recovery_fabric_name'")
            __props__['source_recovery_fabric_name'] = source_recovery_fabric_name
            if target_network_id is None:
                raise TypeError("Missing required property 'target_network_id'")
            __props__['target_network_id'] = target_network_id
            if target_recovery_fabric_name is None:
                raise TypeError("Missing required property 'target_recovery_fabric_name'")
            __props__['target_recovery_fabric_name'] = target_recovery_fabric_name
        super(NetworkMapping, __self__).__init__(
            'azure:recoveryservices/networkMapping:NetworkMapping',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, name=None, recovery_vault_name=None, resource_group_name=None, source_network_id=None, source_recovery_fabric_name=None, target_network_id=None, target_recovery_fabric_name=None):
        """
        Get an existing NetworkMapping resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.
        
        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] name: The name of the network mapping.
        :param pulumi.Input[str] recovery_vault_name: The name of the vault that should be updated.
        :param pulumi.Input[str] resource_group_name: Name of the resource group where the vault that should be updated is located.
        :param pulumi.Input[str] source_network_id: The id of the primary network.
        :param pulumi.Input[str] source_recovery_fabric_name: Specifies the ASR fabric where mapping should be created.
        :param pulumi.Input[str] target_network_id: The id of the recovery network.
        :param pulumi.Input[str] target_recovery_fabric_name: The Azure Site Recovery fabric object corresponding to the recovery Azure region.

        > This content is derived from https://github.com/terraform-providers/terraform-provider-azurerm/blob/master/website/docs/r/recovery_network_mapping.html.markdown.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()
        __props__["name"] = name
        __props__["recovery_vault_name"] = recovery_vault_name
        __props__["resource_group_name"] = resource_group_name
        __props__["source_network_id"] = source_network_id
        __props__["source_recovery_fabric_name"] = source_recovery_fabric_name
        __props__["target_network_id"] = target_network_id
        __props__["target_recovery_fabric_name"] = target_recovery_fabric_name
        return NetworkMapping(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class GetZoneResult:
    """
    A collection of values returned by getZone.
    """
    def __init__(__self__, max_number_of_record_sets=None, name=None, name_servers=None, number_of_record_sets=None, registration_virtual_network_ids=None, resolution_virtual_network_ids=None, resource_group_name=None, tags=None, zone_type=None, id=None):
        if max_number_of_record_sets and not isinstance(max_number_of_record_sets, float):
            raise TypeError("Expected argument 'max_number_of_record_sets' to be a float")
        __self__.max_number_of_record_sets = max_number_of_record_sets
        """
        Maximum number of Records in the zone.
        """
        if name and not isinstance(name, str):
            raise TypeError("Expected argument 'name' to be a str")
        __self__.name = name
        if name_servers and not isinstance(name_servers, list):
            raise TypeError("Expected argument 'name_servers' to be a list")
        __self__.name_servers = name_servers
        """
        A list of values that make up the NS record for the zone.
        """
        if number_of_record_sets and not isinstance(number_of_record_sets, float):
            raise TypeError("Expected argument 'number_of_record_sets' to be a float")
        __self__.number_of_record_sets = number_of_record_sets
        """
        The number of records already in the zone.
        """
        if registration_virtual_network_ids and not isinstance(registration_virtual_network_ids, list):
            raise TypeError("Expected argument 'registration_virtual_network_ids' to be a list")
        __self__.registration_virtual_network_ids = registration_virtual_network_ids
        """
        A list of Virtual Network ID's that register hostnames in this DNS zone.
        """
        if resolution_virtual_network_ids and not isinstance(resolution_virtual_network_ids, list):
            raise TypeError("Expected argument 'resolution_virtual_network_ids' to be a list")
        __self__.resolution_virtual_network_ids = resolution_virtual_network_ids
        """
        A list of Virtual Network ID's that resolve records in this DNS zone.
        """
        if resource_group_name and not isinstance(resource_group_name, str):
            raise TypeError("Expected argument 'resource_group_name' to be a str")
        __self__.resource_group_name = resource_group_name
        if tags and not isinstance(tags, dict):
            raise TypeError("Expected argument 'tags' to be a dict")
        __self__.tags = tags
        """
        A mapping of tags to assign to the EventHub Namespace.
        """
        if zone_type and not isinstance(zone_type, str):
            raise TypeError("Expected argument 'zone_type' to be a str")
        __self__.zone_type = zone_type
        """
        (**Deprecated**) The type of this DNS zone, such as `Public` or `Private`.
        """
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        __self__.id = id
        """
        id is the provider-assigned unique ID for this managed resource.
        """
class AwaitableGetZoneResult(GetZoneResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetZoneResult(
            max_number_of_record_sets=self.max_number_of_record_sets,
            name=self.name,
            name_servers=self.name_servers,
            number_of_record_sets=self.number_of_record_sets,
            registration_virtual_network_ids=self.registration_virtual_network_ids,
            resolution_virtual_network_ids=self.resolution_virtual_network_ids,
            resource_group_name=self.resource_group_name,
            tags=self.tags,
            zone_type=self.zone_type,
            id=self.id)

def get_zone(name=None,resource_group_name=None,opts=None):
    """
    Use this data source to access information about an existing DNS Zone.
    
    :param str name: The name of the DNS Zone.
    :param str resource_group_name: The Name of the Resource Group where the DNS Zone exists.
           If the Name of the Resource Group is not provided, the first DNS Zone from the list of DNS Zones
           in your subscription that matches `name` will be returned.

    > This content is derived from https://github.com/terraform-providers/terraform-provider-azurerm/blob/master/website/docs/d/dns_zone.html.markdown.
    """
    __args__ = dict()

    __args__['name'] = name
    __args__['resourceGroupName'] = resource_group_name
    if opts is None:
        opts = pulumi.InvokeOptions()
    if opts.version is None:
        opts.version = utilities.get_version()
    __ret__ = pulumi.runtime.invoke('azure:dns/getZone:getZone', __args__, opts=opts).value

    return AwaitableGetZoneResult(
        max_number_of_record_sets=__ret__.get('maxNumberOfRecordSets'),
        name=__ret__.get('name'),
        name_servers=__ret__.get('nameServers'),
        number_of_record_sets=__ret__.get('numberOfRecordSets'),
        registration_virtual_network_ids=__ret__.get('registrationVirtualNetworkIds'),
        resolution_virtual_network_ids=__ret__.get('resolutionVirtualNetworkIds'),
        resource_group_name=__ret__.get('resourceGroupName'),
        tags=__ret__.get('tags'),
        zone_type=__ret__.get('zoneType'),
        id=__ret__.get('id'))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class GetRouteTableResult:
    """
    A collection of values returned by getRouteTable.
    """
    def __init__(__self__, location=None, name=None, resource_group_name=None, routes=None, subnets=None, tags=None, id=None):
        if location and not isinstance(location, str):
            raise TypeError("Expected argument 'location' to be a str")
        __self__.location = location
        """
        The Azure Region in which the Route Table exists.
        """
        if name and not isinstance(name, str):
            raise TypeError("Expected argument 'name' to be a str")
        __self__.name = name
        """
        The name of the Route.
        """
        if resource_group_name and not isinstance(resource_group_name, str):
            raise TypeError("Expected argument 'resource_group_name' to be a str")
        __self__.resource_group_name = resource_group_name
        if routes and not isinstance(routes, list):
            raise TypeError("Expected argument 'routes' to be a list")
        __self__.routes = routes
        """
        One or more `route` blocks as documented below.
        """
        if subnets and not isinstance(subnets, list):
            raise TypeError("Expected argument 'subnets' to be a list")
        __self__.subnets = subnets
        """
        The collection of Subnets associated with this route table.
        """
        if tags and not isinstance(tags, dict):
            raise TypeError("Expected argument 'tags' to be a dict")
        __self__.tags = tags
        """
        A mapping of tags assigned to the Route Table.
        """
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        __self__.id = id
        """
        id is the provider-assigned unique ID for this managed resource.
        """
class AwaitableGetRouteTableResult(GetRouteTableResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetRouteTableResult(
            location=self.location,
            name=self.name,
            resource_group_name=self.resource_group_name,
            routes=self.routes,
            subnets=self.subnets,
            tags=self.tags,
            id=self.id)

def get_route_table(name=None,resource_group_name=None,opts=None):
    """
    Use this data source to access information about an existing Route Table.
    
    :param str name: The name of the Route Table.
    :param str resource_group_name: The name of the Resource Group in which the Route Table exists.

    > This content is derived from https://github.com/terraform-providers/terraform-provider-azurerm/blob/master/website/docs/d/route_table.html.markdown.
    """
    __args__ = dict()

    __args__['name'] = name
    __args__['resourceGroupName'] = resource_group_name
    if opts is None:
        opts = pulumi.InvokeOptions()
    if opts.version is None:
        opts.version = utilities.get_version()
    __ret__ = pulumi.runtime.invoke('azure:network/getRouteTable:getRouteTable', __args__, opts=opts).value

    return AwaitableGetRouteTableResult(
        location=__ret__.get('location'),
        name=__ret__.get('name'),
        resource_group_name=__ret__.get('resourceGroupName'),
        routes=__ret__.get('routes'),
        subnets=__ret__.get('subnets'),
        tags=__ret__.get('tags'),
        id=__ret__.get('id'))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from .. import utilities, tables

class Definition(pulumi.CustomResource):
    assignable_scopes: pulumi.Output[list]
    """
    One or more assignable scopes for this Role Definition, such as `/subscriptions/0b1f6471-1bf0-4dda-aec3-111122223333`, `/subscriptions/0b1f6471-1bf0-4dda-aec3-111122223333/resourceGroups/myGroup`, or `/subscriptions/0b1f6471-1bf0-4dda-aec3-111122223333/resourceGroups/myGroup/providers/Microsoft.Compute/virtualMachines/myVM`.
    """
    description: pulumi.Output[str]
    """
    A description of the Role Definition.
    """
    name: pulumi.Output[str]
    """
    The name of the Role Definition. Changing this forces a new resource to be created.
    """
    permissions: pulumi.Output[list]
    """
    A `permissions` block as defined below.
    """
    role_definition_id: pulumi.Output[str]
    """
    A unique UUID/GUID which identifies this role - one will be generated if not specified. Changing this forces a new resource to be created.
    """
    scope: pulumi.Output[str]
    """
    The scope at which the Role Definition applies too, such as `/subscriptions/0b1f6471-1bf0-4dda-aec3-111122223333`, `/subscriptions/0b1f6471-1bf0-4dda-aec3-111122223333/resourceGroups/myGroup`, or `/subscriptions/0b1f6471-1bf0-4dda-aec3-111122223333/resourceGroups/myGroup/providers/Microsoft.Compute/virtualMachines/myVM`. Changing this forces a new resource to be created.
    """
    def __init__(__self__, resource_name, opts=None, assignable_scopes=None, description=None, name=None, permissions=None, role_definition_id=None, scope=None, __name__=None, __opts__=None):
        """
        Manages a custom Role Definition, used to assign Roles to Users/Principals. See ['Understand role definitions'](https://docs.microsoft.com/en-us/azure/role-based-access-control/role-definitions) in the Azure documentation for more details.
        
        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[list] assignable_scopes: One or more assignable scopes for this Role Definition, such as `/subscriptions/0b1f6471-1bf0-4dda-aec3-111122223333`, `/subscriptions/0b1f6471-1bf0-4dda-aec3-111122223333/resourceGroups/myGroup`, or `/subscriptions/0b1f6471-1bf0-4dda-aec3-111122223333/resourceGroups/myGroup/providers/Microsoft.Compute/virtualMachines/myVM`.
        :param pulumi.Input[str] description: A description of the Role Definition.
        :param pulumi.Input[str] name: The name of the Role Definition. Changing this forces a new resource to be created.
        :param pulumi.Input[list] permissions: A `permissions` block as defined below.
        :param pulumi.Input[str] role_definition_id: A unique UUID/GUID which identifies this role - one will be generated if not specified. Changing this forces a new resource to be created.
        :param pulumi.Input[str] scope: The scope at which the Role Definition applies too, such as `/subscriptions/0b1f6471-1bf0-4dda-aec3-111122223333`, `/subscriptions/0b1f6471-1bf0-4dda-aec3-111122223333/resourceGroups/myGroup`, or `/subscriptions/0b1f6471-1bf0-4dda-aec3-111122223333/resourceGroups/myGroup/providers/Microsoft.Compute/virtualMachines/myVM`. Changing this forces a new resource to be created.

        > This content is derived from https://github.com/terraform-providers/terraform-provider-azurerm/blob/master/website/docs/r/role_definition.html.markdown.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if not resource_name:
            raise TypeError('Missing resource name argument (for URN creation)')
        if not isinstance(resource_name, str):
            raise TypeError('Expected resource name to be a string')
        if opts and not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')

        __props__ = dict()

        if assignable_scopes is None:
            raise TypeError("Missing required property 'assignable_scopes'")
        __props__['assignable_scopes'] = assignable_scopes

        __props__['description'] = description

        __props__['name'] = name

        if permissions is None:
            raise TypeError("Missing required property 'permissions'")
        __props__['permissions'] = permissions

        __props__['role_definition_id'] = role_definition_id

        if scope is None:
            raise TypeError("Missing required property 'scope'")
        __props__['scope'] = scope

        if opts is None:
            opts = pulumi.ResourceOptions()
        if opts.version is None:
            opts.version = utilities.get_version()
        alias_opts = pulumi.ResourceOptions(aliases=[pulumi.Alias(type_="azure:role/definition:Definition")])
        opts = alias_opts if opts is None else opts.merge(alias_opts)
        super(Definition, __self__).__init__(
            'azure:authorization/definition:Definition',
            resource_name,
            __props__,
            opts)


    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class Server(pulumi.CustomResource):
    admin_users: pulumi.Output[list]
    """
    List of email addresses of admin users.
    """
    backup_blob_container_uri: pulumi.Output[str]
    """
    URI and SAS token for a blob container to store backups.
    """
    enable_power_bi_service: pulumi.Output[bool]
    """
    Indicates if the Power BI service is allowed to access or not.
    """
    ipv4_firewall_rules: pulumi.Output[list]
    """
    One or more `ipv4_firewall_rule` block(s) as defined below.
    
      * `name` (`str`) - Specifies the name of the firewall rule.
      * `rangeEnd` (`str`) - End of the firewall rule range as IPv4 address.
      * `rangeStart` (`str`) - Start of the firewall rule range as IPv4 address.
    """
    location: pulumi.Output[str]
    """
    The Azure location where the Analysis Services Server exists. Changing this forces a new resource to be created.
    """
    name: pulumi.Output[str]
    """
    Specifies the name of the firewall rule.
    """
    querypool_connection_mode: pulumi.Output[str]
    """
    Controls how the read-write server is used in the query pool. If this values is set to `All` then read-write servers are also used for queries. Otherwise with `ReadOnly` these servers do not participate in query operations.
    """
    resource_group_name: pulumi.Output[str]
    """
    The name of the Resource Group in which the Analysis Services Server should be exist. Changing this forces a new resource to be created.
    """
    server_full_name: pulumi.Output[str]
    """
    The full name of the Analysis Services Server.
    """
    sku: pulumi.Output[str]
    """
    SKU for the Analysis Services Server. Possible values are: `D1`, `B1`, `B2`, `S0`, `S1`, `S2`, `S4`, `S8` and `S9`
    """
    tags: pulumi.Output[dict]
    def __init__(__self__, resource_name, opts=None, admin_users=None, backup_blob_container_uri=None, enable_power_bi_service=None, ipv4_firewall_rules=None, location=None, name=None, querypool_connection_mode=None, resource_group_name=None, sku=None, tags=None, __props__=None, __name__=None, __opts__=None):
        """
        Manages an Analysis Services Server.
        
        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[list] admin_users: List of email addresses of admin users.
        :param pulumi.Input[str] backup_blob_container_uri: URI and SAS token for a blob container to store backups.
        :param pulumi.Input[bool] enable_power_bi_service: Indicates if the Power BI service is allowed to access or not.
        :param pulumi.Input[list] ipv4_firewall_rules: One or more `ipv4_firewall_rule` block(s) as defined below.
        :param pulumi.Input[str] location: The Azure location where the Analysis Services Server exists. Changing this forces a new resource to be created.
        :param pulumi.Input[str] name: Specifies the name of the firewall rule.
        :param pulumi.Input[str] querypool_connection_mode: Controls how the read-write server is used in the query pool. If this values is set to `All` then read-write servers are also used for queries. Otherwise with `ReadOnly` these servers do not participate in query operations.
        :param pulumi.Input[str] resource_group_name: The name of the Resource Group in which the Analysis Services Server should be exist. Changing this forces a new resource to be created.
        :param pulumi.Input[str] sku: SKU for the Analysis Services Server. Possible values are: `D1`, `B1`, `B2`, `S0`, `S1`, `S2`, `S4`, `S8` and `S9`
        
        The **ipv4_firewall_rules** object supports the following:
        
          * `name` (`pulumi.Input[str]`) - Specifies the name of the firewall rule.
          * `rangeEnd` (`pulumi.Input[str]`) - End of the firewall rule range as IPv4 address.
          * `rangeStart` (`pulumi.Input[str]`) - Start of the firewall rule range as IPv4 address.

        > This content is derived from https://github.com/terraform-providers/terraform-provider-azurerm/blob/master/website/docs/r/analysis_services_server.html.markdown.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['admin_users'] = admin_users
            __props__['backup_blob_container_uri'] = backup_blob_container_uri
            __props__['enable_power_bi_service'] = enable_power_bi_service
            __props__['ipv4_firewall_rules'] = ipv4_firewall_rules
            __props__['location'] = location
            __props__['name'] = name
            __props__['querypool_connection_mode'] = querypool_connection_mode
            if resource_group_name is None:
                raise TypeError("Missing required property 'resource_group_name'")
            __props__['resource_group_name'] = resource_group_name
            if sku is None:
                raise TypeError("Missing required property 'sku'")
            __props__['sku'] = sku
            __props__['tags'] = tags
            __props__['server_full_name'] = None
        super(Server, __self__).__init__(
            'azure:analysisservices/server:Server',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, admin_users=None, backup_blob_container_uri=None, enable_power_bi_service=None, ipv4_firewall_rules=None, location=None, name=None, querypool_connection_mode=None, resource_group_name=None, server_full_name=None, sku=None, tags=None):
        """
        Get an existing Server resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.
        
        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[list] admin_users: List of email addresses of admin users.
        :param pulumi.Input[str] backup_blob_container_uri: URI and SAS token for a blob container to store backups.
        :param pulumi.Input[bool] enable_power_bi_service: Indicates if the Power BI service is allowed to access or not.
        :param pulumi.Input[list] ipv4_firewall_rules: One or more `ipv4_firewall_rule` block(s) as defined below.
        :param pulumi.Input[str] location: The Azure location where the Analysis Services Server exists. Changing this forces a new resource to be created.
        :param pulumi.Input[str] name: Specifies the name of the firewall rule.
        :param pulumi.Input[str] querypool_connection_mode: Controls how the read-write server is used in the query pool. If this values is set to `All` then read-write servers are also used for queries. Otherwise with `ReadOnly` these servers do not participate in query operations.
        :param pulumi.Input[str] resource_group_name: The name of the Resource Group in which the Analysis Services Server should be exist. Changing this forces a new resource to be created.
        :param pulumi.Input[str] server_full_name: The full name of the Analysis Services Server.
        :param pulumi.Input[str] sku: SKU for the Analysis Services Server. Possible values are: `D1`, `B1`, `B2`, `S0`, `S1`, `S2`, `S4`, `S8` and `S9`
        
        The **ipv4_firewall_rules** object supports the following:
        
          * `name` (`pulumi.Input[str]`) - Specifies the name of the firewall rule.
          * `rangeEnd` (`pulumi.Input[str]`) - End of the firewall rule range as IPv4 address.
          * `rangeStart` (`pulumi.Input[str]`) - Start of the firewall rule range as IPv4 address.

        > This content is derived from https://github.com/terraform-providers/terraform-provider-azurerm/blob/master/website/docs/r/analysis_services_server.html.markdown.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()
        __props__["admin_users"] = admin_users
        __props__["backup_blob_container_uri"] = backup_blob_container_uri
        __props__["enable_power_bi_service"] = enable_power_bi_service
        __props__["ipv4_firewall_rules"] = ipv4_firewall_rules
        __props__["location"] = location
        __props__["name"] = name
        __props__["querypool_connection_mode"] = querypool_connection_mode
        __props__["resource_group_name"] = resource_group_name
        __props__["server_full_name"] = server_full_name
        __props__["sku"] = sku
        __props__["tags"] = tags
        return Server(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class Namespace(pulumi.CustomResource):
    capacity: pulumi.Output[float]
    """
    Specifies the capacity. When `sku` is `Premium`, capacity can be `1`, `2`, `4` or `8`. When `sku` is `Basic` or `Standard`, capacity can be `0` only.
    """
    default_primary_connection_string: pulumi.Output[str]
    """
    The primary connection string for the authorization
    rule `RootManageSharedAccessKey`.
    """
    default_primary_key: pulumi.Output[str]
    """
    The primary access key for the authorization rule `RootManageSharedAccessKey`.
    """
    default_secondary_connection_string: pulumi.Output[str]
    """
    The secondary connection string for the
    authorization rule `RootManageSharedAccessKey`.
    """
    default_secondary_key: pulumi.Output[str]
    """
    The secondary access key for the authorization rule `RootManageSharedAccessKey`.
    """
    location: pulumi.Output[str]
    """
    Specifies the supported Azure location where the resource exists. Changing this forces a new resource to be created.
    """
    name: pulumi.Output[str]
    """
    Specifies the name of the ServiceBus Namespace resource . Changing this forces a
    new resource to be created.
    """
    resource_group_name: pulumi.Output[str]
    """
    The name of the resource group in which to
    create the namespace.
    """
    sku: pulumi.Output[str]
    """
    Defines which tier to use. Options are basic, standard or premium.
    """
    tags: pulumi.Output[dict]
    """
    A mapping of tags to assign to the resource.
    """
    zone_redundant: pulumi.Output[bool]
    """
    Whether or not this resource is zone redundant. `sku` needs to be `Premium`. Defaults to `false`.
    """
    def __init__(__self__, resource_name, opts=None, capacity=None, location=None, name=None, resource_group_name=None, sku=None, tags=None, zone_redundant=None, __props__=None, __name__=None, __opts__=None):
        """
        Manages a ServiceBus Namespace.
        
        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[float] capacity: Specifies the capacity. When `sku` is `Premium`, capacity can be `1`, `2`, `4` or `8`. When `sku` is `Basic` or `Standard`, capacity can be `0` only.
        :param pulumi.Input[str] location: Specifies the supported Azure location where the resource exists. Changing this forces a new resource to be created.
        :param pulumi.Input[str] name: Specifies the name of the ServiceBus Namespace resource . Changing this forces a
               new resource to be created.
        :param pulumi.Input[str] resource_group_name: The name of the resource group in which to
               create the namespace.
        :param pulumi.Input[str] sku: Defines which tier to use. Options are basic, standard or premium.
        :param pulumi.Input[dict] tags: A mapping of tags to assign to the resource.
        :param pulumi.Input[bool] zone_redundant: Whether or not this resource is zone redundant. `sku` needs to be `Premium`. Defaults to `false`.

        > This content is derived from https://github.com/terraform-providers/terraform-provider-azurerm/blob/master/website/docs/r/servicebus_namespace.html.markdown.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['capacity'] = capacity
            __props__['location'] = location
            __props__['name'] = name
            if resource_group_name is None:
                raise TypeError("Missing required property 'resource_group_name'")
            __props__['resource_group_name'] = resource_group_name
            if sku is None:
                raise TypeError("Missing required property 'sku'")
            __props__['sku'] = sku
            __props__['tags'] = tags
            __props__['zone_redundant'] = zone_redundant
            __props__['default_primary_connection_string'] = None
            __props__['default_primary_key'] = None
            __props__['default_secondary_connection_string'] = None
            __props__['default_secondary_key'] = None
        alias_opts = pulumi.ResourceOptions(aliases=[pulumi.Alias(type_="azure:eventhub/namespace:Namespace")])
        opts = pulumi.ResourceOptions.merge(opts, alias_opts)
        super(Namespace, __self__).__init__(
            'azure:servicebus/namespace:Namespace',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, capacity=None, default_primary_connection_string=None, default_primary_key=None, default_secondary_connection_string=None, default_secondary_key=None, location=None, name=None, resource_group_name=None, sku=None, tags=None, zone_redundant=None):
        """
        Get an existing Namespace resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.
        
        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[float] capacity: Specifies the capacity. When `sku` is `Premium`, capacity can be `1`, `2`, `4` or `8`. When `sku` is `Basic` or `Standard`, capacity can be `0` only.
        :param pulumi.Input[str] default_primary_connection_string: The primary connection string for the authorization
               rule `RootManageSharedAccessKey`.
        :param pulumi.Input[str] default_primary_key: The primary access key for the authorization rule `RootManageSharedAccessKey`.
        :param pulumi.Input[str] default_secondary_connection_string: The secondary connection string for the
               authorization rule `RootManageSharedAccessKey`.
        :param pulumi.Input[str] default_secondary_key: The secondary access key for the authorization rule `RootManageSharedAccessKey`.
        :param pulumi.Input[str] location: Specifies the supported Azure location where the resource exists. Changing this forces a new resource to be created.
        :param pulumi.Input[str] name: Specifies the name of the ServiceBus Namespace resource . Changing this forces a
               new resource to be created.
        :param pulumi.Input[str] resource_group_name: The name of the resource group in which to
               create the namespace.
        :param pulumi.Input[str] sku: Defines which tier to use. Options are basic, standard or premium.
        :param pulumi.Input[dict] tags: A mapping of tags to assign to the resource.
        :param pulumi.Input[bool] zone_redundant: Whether or not this resource is zone redundant. `sku` needs to be `Premium`. Defaults to `false`.

        > This content is derived from https://github.com/terraform-providers/terraform-provider-azurerm/blob/master/website/docs/r/servicebus_namespace.html.markdown.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()
        __props__["capacity"] = capacity
        __props__["default_primary_connection_string"] = default_primary_connection_string
        __props__["default_primary_key"] = default_primary_key
        __props__["default_secondary_connection_string"] = default_secondary_connection_string
        __props__["default_secondary_key"] = default_secondary_key
        __props__["location"] = location
        __props__["name"] = name
        __props__["resource_group_name"] = resource_group_name
        __props__["sku"] = sku
        __props__["tags"] = tags
        __props__["zone_redundant"] = zone_redundant
        return Namespace(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class GetAppServiceResult:
    """
    A collection of values returned by getAppService.
    """
    def __init__(__self__, app_service_plan_id=None, app_settings=None, client_affinity_enabled=None, client_cert_enabled=None, connection_strings=None, default_site_hostname=None, enabled=None, https_only=None, location=None, name=None, outbound_ip_addresses=None, possible_outbound_ip_addresses=None, resource_group_name=None, site_configs=None, site_credentials=None, source_controls=None, tags=None, id=None):
        if app_service_plan_id and not isinstance(app_service_plan_id, str):
            raise TypeError("Expected argument 'app_service_plan_id' to be a str")
        __self__.app_service_plan_id = app_service_plan_id
        """
        The ID of the App Service Plan within which the App Service exists.
        """
        if app_settings and not isinstance(app_settings, dict):
            raise TypeError("Expected argument 'app_settings' to be a dict")
        __self__.app_settings = app_settings
        """
        A key-value pair of App Settings for the App Service.
        """
        if client_affinity_enabled and not isinstance(client_affinity_enabled, bool):
            raise TypeError("Expected argument 'client_affinity_enabled' to be a bool")
        __self__.client_affinity_enabled = client_affinity_enabled
        """
        Does the App Service send session affinity cookies, which route client requests in the same session to the same instance?
        """
        if client_cert_enabled and not isinstance(client_cert_enabled, bool):
            raise TypeError("Expected argument 'client_cert_enabled' to be a bool")
        __self__.client_cert_enabled = client_cert_enabled
        """
        Does the App Service require client certificates for incoming requests?
        """
        if connection_strings and not isinstance(connection_strings, list):
            raise TypeError("Expected argument 'connection_strings' to be a list")
        __self__.connection_strings = connection_strings
        """
        An `connection_string` block as defined below.
        """
        if default_site_hostname and not isinstance(default_site_hostname, str):
            raise TypeError("Expected argument 'default_site_hostname' to be a str")
        __self__.default_site_hostname = default_site_hostname
        """
        The Default Hostname associated with the App Service - such as `mysite.azurewebsites.net`
        """
        if enabled and not isinstance(enabled, bool):
            raise TypeError("Expected argument 'enabled' to be a bool")
        __self__.enabled = enabled
        """
        Is the App Service Enabled?
        """
        if https_only and not isinstance(https_only, bool):
            raise TypeError("Expected argument 'https_only' to be a bool")
        __self__.https_only = https_only
        """
        Can the App Service only be accessed via HTTPS?
        """
        if location and not isinstance(location, str):
            raise TypeError("Expected argument 'location' to be a str")
        __self__.location = location
        """
        The Azure location where the App Service exists.
        """
        if name and not isinstance(name, str):
            raise TypeError("Expected argument 'name' to be a str")
        __self__.name = name
        """
        The name of the Connection String.
        """
        if outbound_ip_addresses and not isinstance(outbound_ip_addresses, str):
            raise TypeError("Expected argument 'outbound_ip_addresses' to be a str")
        __self__.outbound_ip_addresses = outbound_ip_addresses
        """
        A comma separated list of outbound IP addresses - such as `52.23.25.3,52.143.43.12`
        """
        if possible_outbound_ip_addresses and not isinstance(possible_outbound_ip_addresses, str):
            raise TypeError("Expected argument 'possible_outbound_ip_addresses' to be a str")
        __self__.possible_outbound_ip_addresses = possible_outbound_ip_addresses
        """
        A comma separated list of outbound IP addresses - such as `52.23.25.3,52.143.43.12,52.143.43.17` - not all of which are necessarily in use. Superset of `outbound_ip_addresses`.
        """
        if resource_group_name and not isinstance(resource_group_name, str):
            raise TypeError("Expected argument 'resource_group_name' to be a str")
        __self__.resource_group_name = resource_group_name
        if site_configs and not isinstance(site_configs, list):
            raise TypeError("Expected argument 'site_configs' to be a list")
        __self__.site_configs = site_configs
        """
        A `site_config` block as defined below.
        """
        if site_credentials and not isinstance(site_credentials, list):
            raise TypeError("Expected argument 'site_credentials' to be a list")
        __self__.site_credentials = site_credentials
        if source_controls and not isinstance(source_controls, list):
            raise TypeError("Expected argument 'source_controls' to be a list")
        __self__.source_controls = source_controls
        if tags and not isinstance(tags, dict):
            raise TypeError("Expected argument 'tags' to be a dict")
        __self__.tags = tags
        """
        A mapping of tags to assign to the resource.
        """
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        __self__.id = id
        """
        id is the provider-assigned unique ID for this managed resource.
        """
class AwaitableGetAppServiceResult(GetAppServiceResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetAppServiceResult(
            app_service_plan_id=self.app_service_plan_id,
            app_settings=self.app_settings,
            client_affinity_enabled=self.client_affinity_enabled,
            client_cert_enabled=self.client_cert_enabled,
            connection_strings=self.connection_strings,
            default_site_hostname=self.default_site_hostname,
            enabled=self.enabled,
            https_only=self.https_only,
            location=self.location,
            name=self.name,
            outbound_ip_addresses=self.outbound_ip_addresses,
            possible_outbound_ip_addresses=self.possible_outbound_ip_addresses,
            resource_group_name=self.resource_group_name,
            site_configs=self.site_configs,
            site_credentials=self.site_credentials,
            source_controls=self.source_controls,
            tags=self.tags,
            id=self.id)

def get_app_service(name=None,resource_group_name=None,opts=None):
    """
    Use this data source to access information about an existing App Service.
    
    :param str name: The name of the App Service.
    :param str resource_group_name: The Name of the Resource Group where the App Service exists.

    > This content is derived from https://github.com/terraform-providers/terraform-provider-azurerm/blob/master/website/docs/d/app_service.html.markdown.
    """
    __args__ = dict()

    __args__['name'] = name
    __args__['resourceGroupName'] = resource_group_name
    if opts is None:
        opts = pulumi.InvokeOptions()
    if opts.version is None:
        opts.version = utilities.get_version()
    __ret__ = pulumi.runtime.invoke('azure:appservice/getAppService:getAppService', __args__, opts=opts).value

    return AwaitableGetAppServiceResult(
        app_service_plan_id=__ret__.get('appServicePlanId'),
        app_settings=__ret__.get('appSettings'),
        client_affinity_enabled=__ret__.get('clientAffinityEnabled'),
        client_cert_enabled=__ret__.get('clientCertEnabled'),
        connection_strings=__ret__.get('connectionStrings'),
        default_site_hostname=__ret__.get('defaultSiteHostname'),
        enabled=__ret__.get('enabled'),
        https_only=__ret__.get('httpsOnly'),
        location=__ret__.get('location'),
        name=__ret__.get('name'),
        outbound_ip_addresses=__ret__.get('outboundIpAddresses'),
        possible_outbound_ip_addresses=__ret__.get('possibleOutboundIpAddresses'),
        resource_group_name=__ret__.get('resourceGroupName'),
        site_configs=__ret__.get('siteConfigs'),
        site_credentials=__ret__.get('siteCredentials'),
        source_controls=__ret__.get('sourceControls'),
        tags=__ret__.get('tags'),
        id=__ret__.get('id'))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class VirtualNetworkGatewayConnection(pulumi.CustomResource):
    authorization_key: pulumi.Output[str]
    """
    The authorization key associated with the
    Express Route Circuit. This field is required only if the type is an
    ExpressRoute connection.
    """
    enable_bgp: pulumi.Output[bool]
    """
    If `true`, BGP (Border Gateway Protocol) is enabled
    for this connection. Defaults to `false`.
    """
    express_route_circuit_id: pulumi.Output[str]
    """
    The ID of the Express Route Circuit
    when creating an ExpressRoute connection (i.e. when `type` is `ExpressRoute`).
    The Express Route Circuit can be in the same or in a different subscription.
    """
    express_route_gateway_bypass: pulumi.Output[bool]
    """
    If `true`, data packets will bypass ExpressRoute Gateway for data forwarding This is only valid for ExpressRoute connections.
    """
    ipsec_policy: pulumi.Output[dict]
    """
    A `ipsec_policy` block which is documented below.
    Only a single policy can be defined for a connection. For details on
    custom policies refer to [the relevant section in the Azure documentation](https://docs.microsoft.com/en-us/azure/vpn-gateway/vpn-gateway-ipsecikepolicy-rm-powershell).
    
      * `dhGroup` (`str`)
      * `ikeEncryption` (`str`)
      * `ikeIntegrity` (`str`)
      * `ipsecEncryption` (`str`)
      * `ipsecIntegrity` (`str`)
      * `pfsGroup` (`str`)
      * `saDatasize` (`float`)
      * `saLifetime` (`float`)
    """
    local_network_gateway_id: pulumi.Output[str]
    """
    The ID of the local network gateway
    when creating Site-to-Site connection (i.e. when `type` is `IPsec`).
    """
    location: pulumi.Output[str]
    """
    The location/region where the connection is
    located. Changing this forces a new resource to be created.
    """
    name: pulumi.Output[str]
    """
    The name of the connection. Changing the name forces a
    new resource to be created.
    """
    peer_virtual_network_gateway_id: pulumi.Output[str]
    """
    The ID of the peer virtual
    network gateway when creating a VNet-to-VNet connection (i.e. when `type`
    is `Vnet2Vnet`). The peer Virtual Network Gateway can be in the same or
    in a different subscription.
    """
    resource_group_name: pulumi.Output[str]
    """
    The name of the resource group in which to
    create the connection Changing the name forces a new resource to be created.
    """
    routing_weight: pulumi.Output[float]
    """
    The routing weight. Defaults to `10`.
    """
    shared_key: pulumi.Output[str]
    """
    The shared IPSec key. A key must be provided if a
    Site-to-Site or VNet-to-VNet connection is created whereas ExpressRoute
    connections do not need a shared key.
    """
    tags: pulumi.Output[dict]
    """
    A mapping of tags to assign to the resource.
    """
    type: pulumi.Output[str]
    """
    The type of connection. Valid options are `IPsec`
    (Site-to-Site), `ExpressRoute` (ExpressRoute), and `Vnet2Vnet` (VNet-to-VNet).
    Each connection type requires different mandatory arguments (refer to the
    examples above). Changing the connection type will force a new connection
    to be created.
    """
    use_policy_based_traffic_selectors: pulumi.Output[bool]
    """
    If `true`, policy-based traffic
    selectors are enabled for this connection. Enabling policy-based traffic
    selectors requires an `ipsec_policy` block. Defaults to `false`.
    """
    virtual_network_gateway_id: pulumi.Output[str]
    """
    The ID of the Virtual Network Gateway
    in which the connection will be created. Changing the gateway forces a new
    resource to be created.
    """
    def __init__(__self__, resource_name, opts=None, authorization_key=None, enable_bgp=None, express_route_circuit_id=None, express_route_gateway_bypass=None, ipsec_policy=None, local_network_gateway_id=None, location=None, name=None, peer_virtual_network_gateway_id=None, resource_group_name=None, routing_weight=None, shared_key=None, tags=None, type=None, use_policy_based_traffic_selectors=None, virtual_network_gateway_id=None, __props__=None, __name__=None, __opts__=None):
        """
        Manages a connection in an existing Virtual Network Gateway.
        
        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] authorization_key: The authorization key associated with the
               Express Route Circuit. This field is required only if the type is an
               ExpressRoute connection.
        :param pulumi.Input[bool] enable_bgp: If `true`, BGP (Border Gateway Protocol) is enabled
               for this connection. Defaults to `false`.
        :param pulumi.Input[str] express_route_circuit_id: The ID of the Express Route Circuit
               when creating an ExpressRoute connection (i.e. when `type` is `ExpressRoute`).
               The Express Route Circuit can be in the same or in a different subscription.
        :param pulumi.Input[bool] express_route_gateway_bypass: If `true`, data packets will bypass ExpressRoute Gateway for data forwarding This is only valid for ExpressRoute connections.
        :param pulumi.Input[dict] ipsec_policy: A `ipsec_policy` block which is documented below.
               Only a single policy can be defined for a connection. For details on
               custom policies refer to [the relevant section in the Azure documentation](https://docs.microsoft.com/en-us/azure/vpn-gateway/vpn-gateway-ipsecikepolicy-rm-powershell).
        :param pulumi.Input[str] local_network_gateway_id: The ID of the local network gateway
               when creating Site-to-Site connection (i.e. when `type` is `IPsec`).
        :param pulumi.Input[str] location: The location/region where the connection is
               located. Changing this forces a new resource to be created.
        :param pulumi.Input[str] name: The name of the connection. Changing the name forces a
               new resource to be created.
        :param pulumi.Input[str] peer_virtual_network_gateway_id: The ID of the peer virtual
               network gateway when creating a VNet-to-VNet connection (i.e. when `type`
               is `Vnet2Vnet`). The peer Virtual Network Gateway can be in the same or
               in a different subscription.
        :param pulumi.Input[str] resource_group_name: The name of the resource group in which to
               create the connection Changing the name forces a new resource to be created.
        :param pulumi.Input[float] routing_weight: The routing weight. Defaults to `10`.
        :param pulumi.Input[str] shared_key: The shared IPSec key. A key must be provided if a
               Site-to-Site or VNet-to-VNet connection is created whereas ExpressRoute
               connections do not need a shared key.
        :param pulumi.Input[dict] tags: A mapping of tags to assign to the resource.
        :param pulumi.Input[str] type: The type of connection. Valid options are `IPsec`
               (Site-to-Site), `ExpressRoute` (ExpressRoute), and `Vnet2Vnet` (VNet-to-VNet).
               Each connection type requires different mandatory arguments (refer to the
               examples above). Changing the connection type will force a new connection
               to be created.
        :param pulumi.Input[bool] use_policy_based_traffic_selectors: If `true`, policy-based traffic
               selectors are enabled for this connection. Enabling policy-based traffic
               selectors requires an `ipsec_policy` block. Defaults to `false`.
        :param pulumi.Input[str] virtual_network_gateway_id: The ID of the Virtual Network Gateway
               in which the connection will be created. Changing the gateway forces a new
               resource to be created.
        
        The **ipsec_policy** object supports the following:
        
          * `dhGroup` (`pulumi.Input[str]`)
          * `ikeEncryption` (`pulumi.Input[str]`)
          * `ikeIntegrity` (`pulumi.Input[str]`)
          * `ipsecEncryption` (`pulumi.Input[str]`)
          * `ipsecIntegrity` (`pulumi.Input[str]`)
          * `pfsGroup` (`pulumi.Input[str]`)
          * `saDatasize` (`pulumi.Input[float]`)
          * `saLifetime` (`pulumi.Input[float]`)

        > This content is derived from https://github.com/terraform-providers/terraform-provider-azurerm/blob/master/website/docs/r/virtual_network_gateway_connection.html.markdown.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['authorization_key'] = authorization_key
            __props__['enable_bgp'] = enable_bgp
            __props__['express_route_circuit_id'] = express_route_circuit_id
            __props__['express_route_gateway_bypass'] = express_route_gateway_bypass
            __props__['ipsec_policy'] = ipsec_policy
            __props__['local_network_gateway_id'] = local_network_gateway_id
            __props__['location'] = location
            __props__['name'] = name
            __props__['peer_virtual_network_gateway_id'] = peer_virtual_network_gateway_id
            if resource_group_name is None:
                raise TypeError("Missing required property 'resource_group_name'")
            __props__['resource_group_name'] = resource_group_name
            __props__['routing_weight'] = routing_weight
            __props__['shared_key'] = shared_key
            __props__['tags'] = tags
            if type is None:
                raise TypeError("Missing required property 'type'")
            __props__['type'] = type
            __props__['use_policy_based_traffic_selectors'] = use_policy_based_traffic_selectors
            if virtual_network_gateway_id is None:
                raise TypeError("Missing required property 'virtual_network_gateway_id'")
            __props__['virtual_network_gateway_id'] = virtual_network_gateway_id
        super(VirtualNetworkGatewayConnection, __self__).__init__(
            'azure:network/virtualNetworkGatewayConnection:VirtualNetworkGatewayConnection',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, authorization_key=None, enable_bgp=None, express_route_circuit_id=None, express_route_gateway_bypass=None, ipsec_policy=None, local_network_gateway_id=None, location=None, name=None, peer_virtual_network_gateway_id=None, resource_group_name=None, routing_weight=None, shared_key=None, tags=None, type=None, use_policy_based_traffic_selectors=None, virtual_network_gateway_id=None):
        """
        Get an existing VirtualNetworkGatewayConnection resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.
        
        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] authorization_key: The authorization key associated with the
               Express Route Circuit. This field is required only if the type is an
               ExpressRoute connection.
        :param pulumi.Input[bool] enable_bgp: If `true`, BGP (Border Gateway Protocol) is enabled
               for this connection. Defaults to `false`.
        :param pulumi.Input[str] express_route_circuit_id: The ID of the Express Route Circuit
               when creating an ExpressRoute connection (i.e. when `type` is `ExpressRoute`).
               The Express Route Circuit can be in the same or in a different subscription.
        :param pulumi.Input[bool] express_route_gateway_bypass: If `true`, data packets will bypass ExpressRoute Gateway for data forwarding This is only valid for ExpressRoute connections.
        :param pulumi.Input[dict] ipsec_policy: A `ipsec_policy` block which is documented below.
               Only a single policy can be defined for a connection. For details on
               custom policies refer to [the relevant section in the Azure documentation](https://docs.microsoft.com/en-us/azure/vpn-gateway/vpn-gateway-ipsecikepolicy-rm-powershell).
        :param pulumi.Input[str] local_network_gateway_id: The ID of the local network gateway
               when creating Site-to-Site connection (i.e. when `type` is `IPsec`).
        :param pulumi.Input[str] location: The location/region where the connection is
               located. Changing this forces a new resource to be created.
        :param pulumi.Input[str] name: The name of the connection. Changing the name forces a
               new resource to be created.
        :param pulumi.Input[str] peer_virtual_network_gateway_id: The ID of the peer virtual
               network gateway when creating a VNet-to-VNet connection (i.e. when `type`
               is `Vnet2Vnet`). The peer Virtual Network Gateway can be in the same or
               in a different subscription.
        :param pulumi.Input[str] resource_group_name: The name of the resource group in which to
               create the connection Changing the name forces a new resource to be created.
        :param pulumi.Input[float] routing_weight: The routing weight. Defaults to `10`.
        :param pulumi.Input[str] shared_key: The shared IPSec key. A key must be provided if a
               Site-to-Site or VNet-to-VNet connection is created whereas ExpressRoute
               connections do not need a shared key.
        :param pulumi.Input[dict] tags: A mapping of tags to assign to the resource.
        :param pulumi.Input[str] type: The type of connection. Valid options are `IPsec`
               (Site-to-Site), `ExpressRoute` (ExpressRoute), and `Vnet2Vnet` (VNet-to-VNet).
               Each connection type requires different mandatory arguments (refer to the
               examples above). Changing the connection type will force a new connection
               to be created.
        :param pulumi.Input[bool] use_policy_based_traffic_selectors: If `true`, policy-based traffic
               selectors are enabled for this connection. Enabling policy-based traffic
               selectors requires an `ipsec_policy` block. Defaults to `false`.
        :param pulumi.Input[str] virtual_network_gateway_id: The ID of the Virtual Network Gateway
               in which the connection will be created. Changing the gateway forces a new
               resource to be created.
        
        The **ipsec_policy** object supports the following:
        
          * `dhGroup` (`pulumi.Input[str]`)
          * `ikeEncryption` (`pulumi.Input[str]`)
          * `ikeIntegrity` (`pulumi.Input[str]`)
          * `ipsecEncryption` (`pulumi.Input[str]`)
          * `ipsecIntegrity` (`pulumi.Input[str]`)
          * `pfsGroup` (`pulumi.Input[str]`)
          * `saDatasize` (`pulumi.Input[float]`)
          * `saLifetime` (`pulumi.Input[float]`)

        > This content is derived from https://github.com/terraform-providers/terraform-provider-azurerm/blob/master/website/docs/r/virtual_network_gateway_connection.html.markdown.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()
        __props__["authorization_key"] = authorization_key
        __props__["enable_bgp"] = enable_bgp
        __props__["express_route_circuit_id"] = express_route_circuit_id
        __props__["express_route_gateway_bypass"] = express_route_gateway_bypass
        __props__["ipsec_policy"] = ipsec_policy
        __props__["local_network_gateway_id"] = local_network_gateway_id
        __props__["location"] = location
        __props__["name"] = name
        __props__["peer_virtual_network_gateway_id"] = peer_virtual_network_gateway_id
        __props__["resource_group_name"] = resource_group_name
        __props__["routing_weight"] = routing_weight
        __props__["shared_key"] = shared_key
        __props__["tags"] = tags
        __props__["type"] = type
        __props__["use_policy_based_traffic_selectors"] = use_policy_based_traffic_selectors
        __props__["virtual_network_gateway_id"] = virtual_network_gateway_id
        return VirtualNetworkGatewayConnection(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class Setting(pulumi.CustomResource):
    enabled: pulumi.Output[bool]
    """
    Specifies whether automatic scaling is enabled for the target resource. Defaults to `true`.
    """
    location: pulumi.Output[str]
    """
    Specifies the supported Azure location where the AutoScale Setting should exist. Changing this forces a new resource to be created.
    """
    name: pulumi.Output[str]
    """
    The name of the AutoScale Setting. Changing this forces a new resource to be created.
    """
    notification: pulumi.Output[dict]
    """
    Specifies a `notification` block as defined below.
    
      * `email` (`dict`)
    
        * `customEmails` (`list`)
        * `sendToSubscriptionAdministrator` (`bool`)
        * `sendToSubscriptionCoAdministrator` (`bool`)
    
      * `webhooks` (`list`)
    
        * `properties` (`dict`)
        * `serviceUri` (`str`)
    """
    profiles: pulumi.Output[list]
    """
    Specifies one or more (up to 20) `profile` blocks as defined below.
    
      * `capacity` (`dict`)
    
        * `default` (`float`)
        * `maximum` (`float`)
        * `minimum` (`float`)
    
      * `fixedDate` (`dict`)
    
        * `end` (`str`)
        * `start` (`str`)
        * `timezone` (`str`)
    
      * `name` (`str`) - The name of the AutoScale Setting. Changing this forces a new resource to be created.
      * `recurrence` (`dict`)
    
        * `days` (`list`)
        * `hours` (`float`)
        * `minutes` (`float`)
        * `timezone` (`str`)
    
      * `rules` (`list`)
    
        * `metricTrigger` (`dict`)
    
          * `metricName` (`str`)
          * `metricResourceId` (`str`)
          * `operator` (`str`)
          * `statistic` (`str`)
          * `threshold` (`float`)
          * `timeAggregation` (`str`)
          * `timeGrain` (`str`)
          * `timeWindow` (`str`)
    
        * `scaleAction` (`dict`)
    
          * `cooldown` (`str`)
          * `direction` (`str`)
          * `type` (`str`)
          * `value` (`float`)
    """
    resource_group_name: pulumi.Output[str]
    """
    The name of the Resource Group in the AutoScale Setting should be created. Changing this forces a new resource to be created.
    """
    tags: pulumi.Output[dict]
    """
    A mapping of tags to assign to the resource.
    """
    target_resource_id: pulumi.Output[str]
    """
    Specifies the resource ID of the resource that the autoscale setting should be added to.
    """
    def __init__(__self__, resource_name, opts=None, enabled=None, location=None, name=None, notification=None, profiles=None, resource_group_name=None, tags=None, target_resource_id=None, __props__=None, __name__=None, __opts__=None):
        """
        Manages an AutoScale Setting which can be applied to Virtual Machine Scale Sets, App Services and other scalable resources.
        
        > **NOTE:** This resource has been deprecated in favour of the `monitoring.AutoscaleSetting` resource and will be removed in the next major version of the AzureRM Provider. The new resource shares the same fields as this one, and information on migrating across can be found in this guide.
        
        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[bool] enabled: Specifies whether automatic scaling is enabled for the target resource. Defaults to `true`.
        :param pulumi.Input[str] location: Specifies the supported Azure location where the AutoScale Setting should exist. Changing this forces a new resource to be created.
        :param pulumi.Input[str] name: The name of the AutoScale Setting. Changing this forces a new resource to be created.
        :param pulumi.Input[dict] notification: Specifies a `notification` block as defined below.
        :param pulumi.Input[list] profiles: Specifies one or more (up to 20) `profile` blocks as defined below.
        :param pulumi.Input[str] resource_group_name: The name of the Resource Group in the AutoScale Setting should be created. Changing this forces a new resource to be created.
        :param pulumi.Input[dict] tags: A mapping of tags to assign to the resource.
        :param pulumi.Input[str] target_resource_id: Specifies the resource ID of the resource that the autoscale setting should be added to.
        
        The **notification** object supports the following:
        
          * `email` (`pulumi.Input[dict]`)
        
            * `customEmails` (`pulumi.Input[list]`)
            * `sendToSubscriptionAdministrator` (`pulumi.Input[bool]`)
            * `sendToSubscriptionCoAdministrator` (`pulumi.Input[bool]`)
        
          * `webhooks` (`pulumi.Input[list]`)
        
            * `properties` (`pulumi.Input[dict]`)
            * `serviceUri` (`pulumi.Input[str]`)
        
        The **profiles** object supports the following:
        
          * `capacity` (`pulumi.Input[dict]`)
        
            * `default` (`pulumi.Input[float]`)
            * `maximum` (`pulumi.Input[float]`)
            * `minimum` (`pulumi.Input[float]`)
        
          * `fixedDate` (`pulumi.Input[dict]`)
        
            * `end` (`pulumi.Input[str]`)
            * `start` (`pulumi.Input[str]`)
            * `timezone` (`pulumi.Input[str]`)
        
          * `name` (`pulumi.Input[str]`) - The name of the AutoScale Setting. Changing this forces a new resource to be created.
          * `recurrence` (`pulumi.Input[dict]`)
        
            * `days` (`pulumi.Input[list]`)
            * `hours` (`pulumi.Input[float]`)
            * `minutes` (`pulumi.Input[float]`)
            * `timezone` (`pulumi.Input[str]`)
        
          * `rules` (`pulumi.Input[list]`)
        
            * `metricTrigger` (`pulumi.Input[dict]`)
        
              * `metricName` (`pulumi.Input[str]`)
              * `metricResourceId` (`pulumi.Input[str]`)
              * `operator` (`pulumi.Input[str]`)
              * `statistic` (`pulumi.Input[str]`)
              * `threshold` (`pulumi.Input[float]`)
              * `timeAggregation` (`pulumi.Input[str]`)
              * `timeGrain` (`pulumi.Input[str]`)
              * `timeWindow` (`pulumi.Input[str]`)
        
            * `scaleAction` (`pulumi.Input[dict]`)
        
              * `cooldown` (`pulumi.Input[str]`)
              * `direction` (`pulumi.Input[str]`)
              * `type` (`pulumi.Input[str]`)
              * `value` (`pulumi.Input[float]`)

        > This content is derived from https://github.com/terraform-providers/terraform-provider-azurerm/blob/master/website/docs/r/autoscale_setting.html.markdown.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['enabled'] = enabled
            __props__['location'] = location
            __props__['name'] = name
            __props__['notification'] = notification
            if profiles is None:
                raise TypeError("Missing required property 'profiles'")
            __props__['profiles'] = profiles
            if resource_group_name is None:
                raise TypeError("Missing required property 'resource_group_name'")
            __props__['resource_group_name'] = resource_group_name
            __props__['tags'] = tags
            if target_resource_id is None:
                raise TypeError("Missing required property 'target_resource_id'")
            __props__['target_resource_id'] = target_resource_id
        super(Setting, __self__).__init__(
            'azure:autoscale/setting:Setting',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, enabled=None, location=None, name=None, notification=None, profiles=None, resource_group_name=None, tags=None, target_resource_id=None):
        """
        Get an existing Setting resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.
        
        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[bool] enabled: Specifies whether automatic scaling is enabled for the target resource. Defaults to `true`.
        :param pulumi.Input[str] location: Specifies the supported Azure location where the AutoScale Setting should exist. Changing this forces a new resource to be created.
        :param pulumi.Input[str] name: The name of the AutoScale Setting. Changing this forces a new resource to be created.
        :param pulumi.Input[dict] notification: Specifies a `notification` block as defined below.
        :param pulumi.Input[list] profiles: Specifies one or more (up to 20) `profile` blocks as defined below.
        :param pulumi.Input[str] resource_group_name: The name of the Resource Group in the AutoScale Setting should be created. Changing this forces a new resource to be created.
        :param pulumi.Input[dict] tags: A mapping of tags to assign to the resource.
        :param pulumi.Input[str] target_resource_id: Specifies the resource ID of the resource that the autoscale setting should be added to.
        
        The **notification** object supports the following:
        
          * `email` (`pulumi.Input[dict]`)
        
            * `customEmails` (`pulumi.Input[list]`)
            * `sendToSubscriptionAdministrator` (`pulumi.Input[bool]`)
            * `sendToSubscriptionCoAdministrator` (`pulumi.Input[bool]`)
        
          * `webhooks` (`pulumi.Input[list]`)
        
            * `properties` (`pulumi.Input[dict]`)
            * `serviceUri` (`pulumi.Input[str]`)
        
        The **profiles** object supports the following:
        
          * `capacity` (`pulumi.Input[dict]`)
        
            * `default` (`pulumi.Input[float]`)
            * `maximum` (`pulumi.Input[float]`)
            * `minimum` (`pulumi.Input[float]`)
        
          * `fixedDate` (`pulumi.Input[dict]`)
        
            * `end` (`pulumi.Input[str]`)
            * `start` (`pulumi.Input[str]`)
            * `timezone` (`pulumi.Input[str]`)
        
          * `name` (`pulumi.Input[str]`) - The name of the AutoScale Setting. Changing this forces a new resource to be created.
          * `recurrence` (`pulumi.Input[dict]`)
        
            * `days` (`pulumi.Input[list]`)
            * `hours` (`pulumi.Input[float]`)
            * `minutes` (`pulumi.Input[float]`)
            * `timezone` (`pulumi.Input[str]`)
        
          * `rules` (`pulumi.Input[list]`)
        
            * `metricTrigger` (`pulumi.Input[dict]`)
        
              * `metricName` (`pulumi.Input[str]`)
              * `metricResourceId` (`pulumi.Input[str]`)
              * `operator` (`pulumi.Input[str]`)
              * `statistic` (`pulumi.Input[str]`)
              * `threshold` (`pulumi.Input[float]`)
              * `timeAggregation` (`pulumi.Input[str]`)
              * `timeGrain` (`pulumi.Input[str]`)
              * `timeWindow` (`pulumi.Input[str]`)
        
            * `scaleAction` (`pulumi.Input[dict]`)
        
              * `cooldown` (`pulumi.Input[str]`)
              * `direction` (`pulumi.Input[str]`)
              * `type` (`pulumi.Input[str]`)
              * `value` (`pulumi.Input[float]`)

        > This content is derived from https://github.com/terraform-providers/terraform-provider-azurerm/blob/master/website/docs/r/autoscale_setting.html.markdown.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()
        __props__["enabled"] = enabled
        __props__["location"] = location
        __props__["name"] = name
        __props__["notification"] = notification
        __props__["profiles"] = profiles
        __props__["resource_group_name"] = resource_group_name
        __props__["tags"] = tags
        __props__["target_resource_id"] = target_resource_id
        return Setting(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class VirtualNetwork(pulumi.CustomResource):
    address_spaces: pulumi.Output[list]
    """
    The address space that is used the virtual
    network. You can supply more than one address space. Changing this forces
    a new resource to be created.
    """
    ddos_protection_plan: pulumi.Output[dict]
    """
    A `ddos_protection_plan` block as documented below.
    
      * `enable` (`bool`)
      * `id` (`str`) - The ID of this subnet.
    """
    dns_servers: pulumi.Output[list]
    """
    List of IP addresses of DNS servers
    """
    location: pulumi.Output[str]
    """
    The location/region where the virtual network is
    created. Changing this forces a new resource to be created.
    """
    name: pulumi.Output[str]
    """
    The name of the virtual network. Changing this forces a
    new resource to be created.
    """
    resource_group_name: pulumi.Output[str]
    """
    The name of the resource group in which to
    create the virtual network.
    """
    subnets: pulumi.Output[list]
    """
    Can be specified multiple times to define multiple
    subnets. Each `subnet` block supports fields documented below.
    
      * `address_prefix` (`str`)
      * `id` (`str`) - The ID of this subnet.
      * `name` (`str`) - The name of the virtual network. Changing this forces a
        new resource to be created.
      * `securityGroup` (`str`)
    """
    tags: pulumi.Output[dict]
    """
    A mapping of tags to assign to the resource.
    """
    def __init__(__self__, resource_name, opts=None, address_spaces=None, ddos_protection_plan=None, dns_servers=None, location=None, name=None, resource_group_name=None, subnets=None, tags=None, __props__=None, __name__=None, __opts__=None):
        """
        Manages a virtual network including any configured subnets. Each subnet can
        optionally be configured with a security group to be associated with the subnet.
        
        > **NOTE on Virtual Networks and Subnet's:** This provider currently
        provides both a standalone Subnet resource, and allows for Subnets to be defined in-line within the Virtual Network resource.
        At this time you cannot use a Virtual Network with in-line Subnets in conjunction with any Subnet resources. Doing so will cause a conflict of Subnet configurations and will overwrite Subnet's.
        
        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[list] address_spaces: The address space that is used the virtual
               network. You can supply more than one address space. Changing this forces
               a new resource to be created.
        :param pulumi.Input[dict] ddos_protection_plan: A `ddos_protection_plan` block as documented below.
        :param pulumi.Input[list] dns_servers: List of IP addresses of DNS servers
        :param pulumi.Input[str] location: The location/region where the virtual network is
               created. Changing this forces a new resource to be created.
        :param pulumi.Input[str] name: The name of the virtual network. Changing this forces a
               new resource to be created.
        :param pulumi.Input[str] resource_group_name: The name of the resource group in which to
               create the virtual network.
        :param pulumi.Input[list] subnets: Can be specified multiple times to define multiple
               subnets. Each `subnet` block supports fields documented below.
        :param pulumi.Input[dict] tags: A mapping of tags to assign to the resource.
        
        The **ddos_protection_plan** object supports the following:
        
          * `enable` (`pulumi.Input[bool]`)
          * `id` (`pulumi.Input[str]`) - The ID of this subnet.
        
        The **subnets** object supports the following:
        
          * `address_prefix` (`pulumi.Input[str]`)
          * `id` (`pulumi.Input[str]`) - The ID of this subnet.
          * `name` (`pulumi.Input[str]`) - The name of the virtual network. Changing this forces a
            new resource to be created.
          * `securityGroup` (`pulumi.Input[str]`)

        > This content is derived from https://github.com/terraform-providers/terraform-provider-azurerm/blob/master/website/docs/r/virtual_network.html.markdown.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            if address_spaces is None:
                raise TypeError("Missing required property 'address_spaces'")
            __props__['address_spaces'] = address_spaces
            __props__['ddos_protection_plan'] = ddos_protection_plan
            __props__['dns_servers'] = dns_servers
            __props__['location'] = location
            __props__['name'] = name
            if resource_group_name is None:
                raise TypeError("Missing required property 'resource_group_name'")
            __props__['resource_group_name'] = resource_group_name
            __props__['subnets'] = subnets
            __props__['tags'] = tags
        super(VirtualNetwork, __self__).__init__(
            'azure:network/virtualNetwork:VirtualNetwork',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, address_spaces=None, ddos_protection_plan=None, dns_servers=None, location=None, name=None, resource_group_name=None, subnets=None, tags=None):
        """
        Get an existing VirtualNetwork resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.
        
        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[list] address_spaces: The address space that is used the virtual
               network. You can supply more than one address space. Changing this forces
               a new resource to be created.
        :param pulumi.Input[dict] ddos_protection_plan: A `ddos_protection_plan` block as documented below.
        :param pulumi.Input[list] dns_servers: List of IP addresses of DNS servers
        :param pulumi.Input[str] location: The location/region where the virtual network is
               created. Changing this forces a new resource to be created.
        :param pulumi.Input[str] name: The name of the virtual network. Changing this forces a
               new resource to be created.
        :param pulumi.Input[str] resource_group_name: The name of the resource group in which to
               create the virtual network.
        :param pulumi.Input[list] subnets: Can be specified multiple times to define multiple
               subnets. Each `subnet` block supports fields documented below.
        :param pulumi.Input[dict] tags: A mapping of tags to assign to the resource.
        
        The **ddos_protection_plan** object supports the following:
        
          * `enable` (`pulumi.Input[bool]`)
          * `id` (`pulumi.Input[str]`) - The ID of this subnet.
        
        The **subnets** object supports the following:
        
          * `address_prefix` (`pulumi.Input[str]`)
          * `id` (`pulumi.Input[str]`) - The ID of this subnet.
          * `name` (`pulumi.Input[str]`) - The name of the virtual network. Changing this forces a
            new resource to be created.
          * `securityGroup` (`pulumi.Input[str]`)

        > This content is derived from https://github.com/terraform-providers/terraform-provider-azurerm/blob/master/website/docs/r/virtual_network.html.markdown.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()
        __props__["address_spaces"] = address_spaces
        __props__["ddos_protection_plan"] = ddos_protection_plan
        __props__["dns_servers"] = dns_servers
        __props__["location"] = location
        __props__["name"] = name
        __props__["resource_group_name"] = resource_group_name
        __props__["subnets"] = subnets
        __props__["tags"] = tags
        return VirtualNetwork(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class GetCertificateOrderResult:
    """
    A collection of values returned by getCertificateOrder.
    """
    def __init__(__self__, app_service_certificate_not_renewable_reasons=None, auto_renew=None, certificates=None, csr=None, distinguished_name=None, domain_verification_token=None, expiration_time=None, intermediate_thumbprint=None, is_private_key_external=None, key_size=None, location=None, name=None, product_type=None, resource_group_name=None, root_thumbprint=None, signed_certificate_thumbprint=None, status=None, tags=None, validity_in_years=None, id=None):
        if app_service_certificate_not_renewable_reasons and not isinstance(app_service_certificate_not_renewable_reasons, list):
            raise TypeError("Expected argument 'app_service_certificate_not_renewable_reasons' to be a list")
        __self__.app_service_certificate_not_renewable_reasons = app_service_certificate_not_renewable_reasons
        """
        Reasons why App Service Certificate is not renewable at the current moment.
        """
        if auto_renew and not isinstance(auto_renew, bool):
            raise TypeError("Expected argument 'auto_renew' to be a bool")
        __self__.auto_renew = auto_renew
        """
        true if the certificate should be automatically renewed when it expires; otherwise, false.
        """
        if certificates and not isinstance(certificates, list):
            raise TypeError("Expected argument 'certificates' to be a list")
        __self__.certificates = certificates
        """
        State of the Key Vault secret. A `certificates` block as defined below.
        """
        if csr and not isinstance(csr, str):
            raise TypeError("Expected argument 'csr' to be a str")
        __self__.csr = csr
        """
        Last CSR that was created for this order.
        """
        if distinguished_name and not isinstance(distinguished_name, str):
            raise TypeError("Expected argument 'distinguished_name' to be a str")
        __self__.distinguished_name = distinguished_name
        """
        The Distinguished Name for the App Service Certificate Order.
        """
        if domain_verification_token and not isinstance(domain_verification_token, str):
            raise TypeError("Expected argument 'domain_verification_token' to be a str")
        __self__.domain_verification_token = domain_verification_token
        """
        Domain verification token.
        """
        if expiration_time and not isinstance(expiration_time, str):
            raise TypeError("Expected argument 'expiration_time' to be a str")
        __self__.expiration_time = expiration_time
        """
        Certificate expiration time.
        """
        if intermediate_thumbprint and not isinstance(intermediate_thumbprint, str):
            raise TypeError("Expected argument 'intermediate_thumbprint' to be a str")
        __self__.intermediate_thumbprint = intermediate_thumbprint
        """
        Certificate thumbprint intermediate certificate.
        """
        if is_private_key_external and not isinstance(is_private_key_external, bool):
            raise TypeError("Expected argument 'is_private_key_external' to be a bool")
        __self__.is_private_key_external = is_private_key_external
        """
        Whether the private key is external or not.
        """
        if key_size and not isinstance(key_size, float):
            raise TypeError("Expected argument 'key_size' to be a float")
        __self__.key_size = key_size
        """
        Certificate key size.
        """
        if location and not isinstance(location, str):
            raise TypeError("Expected argument 'location' to be a str")
        __self__.location = location
        """
        The Azure location where the App Service exists.
        """
        if name and not isinstance(name, str):
            raise TypeError("Expected argument 'name' to be a str")
        __self__.name = name
        if product_type and not isinstance(product_type, str):
            raise TypeError("Expected argument 'product_type' to be a str")
        __self__.product_type = product_type
        """
        Certificate product type, such as `Standard` or `WildCard`.
        """
        if resource_group_name and not isinstance(resource_group_name, str):
            raise TypeError("Expected argument 'resource_group_name' to be a str")
        __self__.resource_group_name = resource_group_name
        if root_thumbprint and not isinstance(root_thumbprint, str):
            raise TypeError("Expected argument 'root_thumbprint' to be a str")
        __self__.root_thumbprint = root_thumbprint
        """
        Certificate thumbprint for root certificate.
        """
        if signed_certificate_thumbprint and not isinstance(signed_certificate_thumbprint, str):
            raise TypeError("Expected argument 'signed_certificate_thumbprint' to be a str")
        __self__.signed_certificate_thumbprint = signed_certificate_thumbprint
        """
        Certificate thumbprint for signed certificate.
        """
        if status and not isinstance(status, str):
            raise TypeError("Expected argument 'status' to be a str")
        __self__.status = status
        """
        Current order status.
        """
        if tags and not isinstance(tags, dict):
            raise TypeError("Expected argument 'tags' to be a dict")
        __self__.tags = tags
        """
        A mapping of tags to assign to the resource.
        """
        if validity_in_years and not isinstance(validity_in_years, float):
            raise TypeError("Expected argument 'validity_in_years' to be a float")
        __self__.validity_in_years = validity_in_years
        """
        Duration in years (must be between 1 and 3).
        """
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        __self__.id = id
        """
        id is the provider-assigned unique ID for this managed resource.
        """
class AwaitableGetCertificateOrderResult(GetCertificateOrderResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetCertificateOrderResult(
            app_service_certificate_not_renewable_reasons=self.app_service_certificate_not_renewable_reasons,
            auto_renew=self.auto_renew,
            certificates=self.certificates,
            csr=self.csr,
            distinguished_name=self.distinguished_name,
            domain_verification_token=self.domain_verification_token,
            expiration_time=self.expiration_time,
            intermediate_thumbprint=self.intermediate_thumbprint,
            is_private_key_external=self.is_private_key_external,
            key_size=self.key_size,
            location=self.location,
            name=self.name,
            product_type=self.product_type,
            resource_group_name=self.resource_group_name,
            root_thumbprint=self.root_thumbprint,
            signed_certificate_thumbprint=self.signed_certificate_thumbprint,
            status=self.status,
            tags=self.tags,
            validity_in_years=self.validity_in_years,
            id=self.id)

def get_certificate_order(name=None,resource_group_name=None,opts=None):
    """
    Use this data source to access information about an existing App Service Certificate Order.
    
    :param str name: The name of the App Service.
    :param str resource_group_name: The Name of the Resource Group where the App Service exists.

    > This content is derived from https://github.com/terraform-providers/terraform-provider-azurerm/blob/master/website/docs/d/app_service_certificate_order.html.markdown.
    """
    __args__ = dict()

    __args__['name'] = name
    __args__['resourceGroupName'] = resource_group_name
    if opts is None:
        opts = pulumi.InvokeOptions()
    if opts.version is None:
        opts.version = utilities.get_version()
    __ret__ = pulumi.runtime.invoke('azure:appservice/getCertificateOrder:getCertificateOrder', __args__, opts=opts).value

    return AwaitableGetCertificateOrderResult(
        app_service_certificate_not_renewable_reasons=__ret__.get('appServiceCertificateNotRenewableReasons'),
        auto_renew=__ret__.get('autoRenew'),
        certificates=__ret__.get('certificates'),
        csr=__ret__.get('csr'),
        distinguished_name=__ret__.get('distinguishedName'),
        domain_verification_token=__ret__.get('domainVerificationToken'),
        expiration_time=__ret__.get('expirationTime'),
        intermediate_thumbprint=__ret__.get('intermediateThumbprint'),
        is_private_key_external=__ret__.get('isPrivateKeyExternal'),
        key_size=__ret__.get('keySize'),
        location=__ret__.get('location'),
        name=__ret__.get('name'),
        product_type=__ret__.get('productType'),
        resource_group_name=__ret__.get('resourceGroupName'),
        root_thumbprint=__ret__.get('rootThumbprint'),
        signed_certificate_thumbprint=__ret__.get('signedCertificateThumbprint'),
        status=__ret__.get('status'),
        tags=__ret__.get('tags'),
        validity_in_years=__ret__.get('validityInYears'),
        id=__ret__.get('id'))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class ApplicationGateway(pulumi.CustomResource):
    authentication_certificates: pulumi.Output[list]
    """
    One or more `authentication_certificate` blocks as defined below.
    
      * `data` (`str`)
      * `id` (`str`) - The ID of the Rewrite Rule Set
      * `name` (`str`) - The name of the Application Gateway. Changing this forces a new resource to be created.
    """
    autoscale_configuration: pulumi.Output[dict]
    """
    A `autoscale_configuration` block as defined below.
    
      * `maxCapacity` (`float`)
      * `minCapacity` (`float`)
    """
    backend_address_pools: pulumi.Output[list]
    """
    One or more `backend_address_pool` blocks as defined below.
    
      * `fqdnLists` (`list`)
      * `fqdns` (`list`)
      * `id` (`str`) - The ID of the Rewrite Rule Set
      * `ipAddressLists` (`list`)
      * `ipAddresses` (`list`)
      * `name` (`str`) - The name of the Application Gateway. Changing this forces a new resource to be created.
    """
    backend_http_settings: pulumi.Output[list]
    """
    One or more `backend_http_settings` blocks as defined below.
    
      * `affinityCookieName` (`str`)
      * `authentication_certificates` (`list`) - One or more `authentication_certificate` blocks as defined below.
    
        * `id` (`str`) - The ID of the Rewrite Rule Set
        * `name` (`str`) - The name of the Application Gateway. Changing this forces a new resource to be created.
    
      * `connectionDraining` (`dict`)
    
        * `drainTimeoutSec` (`float`)
        * `enabled` (`bool`)
    
      * `cookieBasedAffinity` (`str`)
      * `host_name` (`str`)
      * `id` (`str`) - The ID of the Rewrite Rule Set
      * `name` (`str`) - The name of the Application Gateway. Changing this forces a new resource to be created.
      * `path` (`str`)
      * `pickHostNameFromBackendAddress` (`bool`)
      * `port` (`float`)
      * `probe_id` (`str`) - The ID of the associated Probe.
      * `probeName` (`str`)
      * `protocol` (`str`)
      * `requestTimeout` (`float`)
    """
    custom_error_configurations: pulumi.Output[list]
    """
    One or more `custom_error_configuration` blocks as defined below.
    
      * `customErrorPageUrl` (`str`)
      * `id` (`str`) - The ID of the Rewrite Rule Set
      * `statusCode` (`str`)
    """
    disabled_ssl_protocols: pulumi.Output[list]
    """
    A list of SSL Protocols which should be disabled on this Application Gateway. Possible values are `TLSv1_0`, `TLSv1_1` and `TLSv1_2`.
    > **NOTE:** `disabled_ssl_protocols ` has been deprecated in favour of `disabled_protocols` in the `ssl_policy` block.
    """
    enable_http2: pulumi.Output[bool]
    """
    Is HTTP2 enabled on the application gateway resource? Defaults to `false`.
    """
    frontend_ip_configurations: pulumi.Output[list]
    """
    One or more `frontend_ip_configuration` blocks as defined below.
    
      * `id` (`str`) - The ID of the Rewrite Rule Set
      * `name` (`str`) - The name of the Application Gateway. Changing this forces a new resource to be created.
      * `private_ip_address` (`str`)
      * `privateIpAddressAllocation` (`str`)
      * `publicIpAddressId` (`str`)
      * `subnet_id` (`str`)
    """
    frontend_ports: pulumi.Output[list]
    """
    One or more `frontend_port` blocks as defined below.
    
      * `id` (`str`) - The ID of the Rewrite Rule Set
      * `name` (`str`) - The name of the Application Gateway. Changing this forces a new resource to be created.
      * `port` (`float`)
    """
    gateway_ip_configurations: pulumi.Output[list]
    """
    One or more `gateway_ip_configuration` blocks as defined below.
    
      * `id` (`str`) - The ID of the Rewrite Rule Set
      * `name` (`str`) - The name of the Application Gateway. Changing this forces a new resource to be created.
      * `subnet_id` (`str`)
    """
    http_listeners: pulumi.Output[list]
    """
    One or more `http_listener` blocks as defined below.
    
      * `custom_error_configurations` (`list`) - One or more `custom_error_configuration` blocks as defined below.
    
        * `customErrorPageUrl` (`str`)
        * `id` (`str`) - The ID of the Rewrite Rule Set
        * `statusCode` (`str`)
    
      * `frontend_ip_configuration_id` (`str`) - The ID of the associated Frontend Configuration.
      * `frontend_ip_configuration_name` (`str`)
      * `frontendPortId` (`str`) - The ID of the associated Frontend Port.
      * `frontendPortName` (`str`)
      * `host_name` (`str`)
      * `id` (`str`) - The ID of the Rewrite Rule Set
      * `name` (`str`) - The name of the Application Gateway. Changing this forces a new resource to be created.
      * `protocol` (`str`)
      * `requireSni` (`bool`)
      * `sslCertificateId` (`str`) - The ID of the associated SSL Certificate.
      * `sslCertificateName` (`str`)
    """
    identity: pulumi.Output[dict]
    """
    A `identity` block.
    
      * `identityIds` (`str`)
      * `type` (`str`)
    """
    location: pulumi.Output[str]
    """
    The Azure region where the Application Gateway should exist. Changing this forces a new resource to be created.
    """
    name: pulumi.Output[str]
    """
    The name of the Application Gateway. Changing this forces a new resource to be created.
    """
    probes: pulumi.Output[list]
    """
    One or more `probe` blocks as defined below.
    
      * `host` (`str`)
      * `id` (`str`) - The ID of the Rewrite Rule Set
      * `interval` (`float`)
      * `match` (`dict`)
    
        * `body` (`str`)
        * `statusCodes` (`list`)
    
      * `minimumServers` (`float`)
      * `name` (`str`) - The name of the Application Gateway. Changing this forces a new resource to be created.
      * `path` (`str`)
      * `pickHostNameFromBackendHttpSettings` (`bool`)
      * `protocol` (`str`)
      * `timeout` (`float`)
      * `unhealthyThreshold` (`float`)
    """
    redirect_configurations: pulumi.Output[list]
    """
    A `redirect_configuration` block as defined below.
    
      * `id` (`str`) - The ID of the Rewrite Rule Set
      * `includePath` (`bool`)
      * `includeQueryString` (`bool`)
      * `name` (`str`) - The name of the Application Gateway. Changing this forces a new resource to be created.
      * `redirectType` (`str`)
      * `targetListenerId` (`str`)
      * `targetListenerName` (`str`)
      * `targetUrl` (`str`)
    """
    request_routing_rules: pulumi.Output[list]
    """
    One or more `request_routing_rule` blocks as defined below.
    
      * `backend_address_pool_id` (`str`) - The ID of the associated Backend Address Pool.
      * `backendAddressPoolName` (`str`)
      * `backendHttpSettingsId` (`str`) - The ID of the associated Backend HTTP Settings Configuration.
      * `backendHttpSettingsName` (`str`)
      * `httpListenerId` (`str`) - The ID of the associated HTTP Listener.
      * `httpListenerName` (`str`)
      * `id` (`str`) - The ID of the Rewrite Rule Set
      * `name` (`str`) - The name of the Application Gateway. Changing this forces a new resource to be created.
      * `redirectConfigurationId` (`str`) - The ID of the associated Redirect Configuration.
      * `redirectConfigurationName` (`str`)
      * `rewriteRuleSetId` (`str`) - The ID of the associated Rewrite Rule Set.
      * `rewriteRuleSetName` (`str`)
      * `ruleType` (`str`)
      * `urlPathMapId` (`str`) - The ID of the associated URL Path Map.
      * `urlPathMapName` (`str`)
    """
    resource_group_name: pulumi.Output[str]
    """
    The name of the resource group in which to the Application Gateway should exist. Changing this forces a new resource to be created.
    """
    rewrite_rule_sets: pulumi.Output[list]
    """
    One or more `rewrite_rule_set` blocks as defined below. Only valid for v2 SKUs.
    
      * `id` (`str`) - The ID of the Rewrite Rule Set
      * `name` (`str`) - The name of the Application Gateway. Changing this forces a new resource to be created.
      * `rewriteRules` (`list`)
    
        * `conditions` (`list`)
    
          * `ignoreCase` (`bool`)
          * `negate` (`bool`)
          * `pattern` (`str`)
          * `variable` (`str`)
    
        * `name` (`str`) - The name of the Application Gateway. Changing this forces a new resource to be created.
        * `requestHeaderConfigurations` (`list`)
    
          * `headerName` (`str`)
          * `headerValue` (`str`)
    
        * `responseHeaderConfigurations` (`list`)
    
          * `headerName` (`str`)
          * `headerValue` (`str`)
    
        * `ruleSequence` (`float`)
    """
    sku: pulumi.Output[dict]
    """
    A `sku` block as defined below.
    
      * `capacity` (`float`)
      * `name` (`str`) - The name of the Application Gateway. Changing this forces a new resource to be created.
      * `tier` (`str`)
    """
    ssl_certificates: pulumi.Output[list]
    """
    One or more `ssl_certificate` blocks as defined below.
    
      * `data` (`str`)
      * `id` (`str`) - The ID of the Rewrite Rule Set
      * `name` (`str`) - The name of the Application Gateway. Changing this forces a new resource to be created.
      * `password` (`str`)
      * `publicCertData` (`str`) - The Public Certificate Data associated with the SSL Certificate.
    """
    ssl_policies: pulumi.Output[list]
    """
    a `ssl policy` block as defined below.
    
      * `cipherSuites` (`list`)
      * `disabledProtocols` (`list`)
      * `minProtocolVersion` (`str`)
      * `policyName` (`str`)
      * `policyType` (`str`)
    """
    tags: pulumi.Output[dict]
    """
    A mapping of tags to assign to the resource.
    """
    trusted_root_certificates: pulumi.Output[list]
    """
    One or more `trusted_root_certificate` blocks as defined below.
    
      * `data` (`str`)
      * `id` (`str`) - The ID of the Rewrite Rule Set
      * `name` (`str`) - The name of the Application Gateway. Changing this forces a new resource to be created.
    """
    url_path_maps: pulumi.Output[list]
    """
    One or more `url_path_map` blocks as defined below.
    
      * `defaultBackendAddressPoolId` (`str`) - The ID of the Default Backend Address Pool.
      * `defaultBackendAddressPoolName` (`str`)
      * `defaultBackendHttpSettingsId` (`str`) - The ID of the Default Backend HTTP Settings Collection.
      * `defaultBackendHttpSettingsName` (`str`)
      * `defaultRedirectConfigurationId` (`str`) - The ID of the Default Redirect Configuration.
      * `defaultRedirectConfigurationName` (`str`)
      * `defaultRewriteRuleSetId` (`str`)
      * `defaultRewriteRuleSetName` (`str`)
      * `id` (`str`) - The ID of the Rewrite Rule Set
      * `name` (`str`) - The name of the Application Gateway. Changing this forces a new resource to be created.
      * `pathRules` (`list`) - A list of `path_rule` blocks as defined above.
    
        * `backend_address_pool_id` (`str`) - The ID of the associated Backend Address Pool.
        * `backendAddressPoolName` (`str`)
        * `backendHttpSettingsId` (`str`) - The ID of the associated Backend HTTP Settings Configuration.
        * `backendHttpSettingsName` (`str`)
        * `id` (`str`) - The ID of the Rewrite Rule Set
        * `name` (`str`) - The name of the Application Gateway. Changing this forces a new resource to be created.
        * `paths` (`list`)
        * `redirectConfigurationId` (`str`) - The ID of the associated Redirect Configuration.
        * `redirectConfigurationName` (`str`)
        * `rewriteRuleSetId` (`str`) - The ID of the associated Rewrite Rule Set.
        * `rewriteRuleSetName` (`str`)
    """
    waf_configuration: pulumi.Output[dict]
    """
    A `waf_configuration` block as defined below.
    
      * `disabledRuleGroups` (`list`)
    
        * `ruleGroupName` (`str`)
        * `rules` (`list`)
    
      * `enabled` (`bool`)
      * `exclusions` (`list`)
    
        * `matchVariable` (`str`)
        * `selector` (`str`)
        * `selectorMatchOperator` (`str`)
    
      * `fileUploadLimitMb` (`float`)
      * `firewallMode` (`str`)
      * `maxRequestBodySizeKb` (`float`)
      * `requestBodyCheck` (`bool`)
      * `ruleSetType` (`str`)
      * `ruleSetVersion` (`str`)
    """
    zones: pulumi.Output[list]
    """
    A collection of availability zones to spread the Application Gateway over.
    """
    def __init__(__self__, resource_name, opts=None, authentication_certificates=None, autoscale_configuration=None, backend_address_pools=None, backend_http_settings=None, custom_error_configurations=None, disabled_ssl_protocols=None, enable_http2=None, frontend_ip_configurations=None, frontend_ports=None, gateway_ip_configurations=None, http_listeners=None, identity=None, location=None, name=None, probes=None, redirect_configurations=None, request_routing_rules=None, resource_group_name=None, rewrite_rule_sets=None, sku=None, ssl_certificates=None, ssl_policies=None, tags=None, trusted_root_certificates=None, url_path_maps=None, waf_configuration=None, zones=None, __props__=None, __name__=None, __opts__=None):
        """
        Manages an Application Gateway.
        
        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[list] authentication_certificates: One or more `authentication_certificate` blocks as defined below.
        :param pulumi.Input[dict] autoscale_configuration: A `autoscale_configuration` block as defined below.
        :param pulumi.Input[list] backend_address_pools: One or more `backend_address_pool` blocks as defined below.
        :param pulumi.Input[list] backend_http_settings: One or more `backend_http_settings` blocks as defined below.
        :param pulumi.Input[list] custom_error_configurations: One or more `custom_error_configuration` blocks as defined below.
        :param pulumi.Input[list] disabled_ssl_protocols: A list of SSL Protocols which should be disabled on this Application Gateway. Possible values are `TLSv1_0`, `TLSv1_1` and `TLSv1_2`.
               > **NOTE:** `disabled_ssl_protocols ` has been deprecated in favour of `disabled_protocols` in the `ssl_policy` block.
        :param pulumi.Input[bool] enable_http2: Is HTTP2 enabled on the application gateway resource? Defaults to `false`.
        :param pulumi.Input[list] frontend_ip_configurations: One or more `frontend_ip_configuration` blocks as defined below.
        :param pulumi.Input[list] frontend_ports: One or more `frontend_port` blocks as defined below.
        :param pulumi.Input[list] gateway_ip_configurations: One or more `gateway_ip_configuration` blocks as defined below.
        :param pulumi.Input[list] http_listeners: One or more `http_listener` blocks as defined below.
        :param pulumi.Input[dict] identity: A `identity` block.
        :param pulumi.Input[str] location: The Azure region where the Application Gateway should exist. Changing this forces a new resource to be created.
        :param pulumi.Input[str] name: The name of the Application Gateway. Changing this forces a new resource to be created.
        :param pulumi.Input[list] probes: One or more `probe` blocks as defined below.
        :param pulumi.Input[list] redirect_configurations: A `redirect_configuration` block as defined below.
        :param pulumi.Input[list] request_routing_rules: One or more `request_routing_rule` blocks as defined below.
        :param pulumi.Input[str] resource_group_name: The name of the resource group in which to the Application Gateway should exist. Changing this forces a new resource to be created.
        :param pulumi.Input[list] rewrite_rule_sets: One or more `rewrite_rule_set` blocks as defined below. Only valid for v2 SKUs.
        :param pulumi.Input[dict] sku: A `sku` block as defined below.
        :param pulumi.Input[list] ssl_certificates: One or more `ssl_certificate` blocks as defined below.
        :param pulumi.Input[list] ssl_policies: a `ssl policy` block as defined below.
        :param pulumi.Input[dict] tags: A mapping of tags to assign to the resource.
        :param pulumi.Input[list] trusted_root_certificates: One or more `trusted_root_certificate` blocks as defined below.
        :param pulumi.Input[list] url_path_maps: One or more `url_path_map` blocks as defined below.
        :param pulumi.Input[dict] waf_configuration: A `waf_configuration` block as defined below.
        :param pulumi.Input[list] zones: A collection of availability zones to spread the Application Gateway over.
        
        The **authentication_certificates** object supports the following:
        
          * `data` (`pulumi.Input[str]`)
          * `id` (`pulumi.Input[str]`) - The ID of the Rewrite Rule Set
          * `name` (`pulumi.Input[str]`) - The name of the Application Gateway. Changing this forces a new resource to be created.
        
        The **autoscale_configuration** object supports the following:
        
          * `maxCapacity` (`pulumi.Input[float]`)
          * `minCapacity` (`pulumi.Input[float]`)
        
        The **backend_address_pools** object supports the following:
        
          * `fqdnLists` (`pulumi.Input[list]`)
          * `fqdns` (`pulumi.Input[list]`)
          * `id` (`pulumi.Input[str]`) - The ID of the Rewrite Rule Set
          * `ipAddressLists` (`pulumi.Input[list]`)
          * `ipAddresses` (`pulumi.Input[list]`)
          * `name` (`pulumi.Input[str]`) - The name of the Application Gateway. Changing this forces a new resource to be created.
        
        The **backend_http_settings** object supports the following:
        
          * `affinityCookieName` (`pulumi.Input[str]`)
          * `authentication_certificates` (`pulumi.Input[list]`) - One or more `authentication_certificate` blocks as defined below.
        
            * `id` (`pulumi.Input[str]`) - The ID of the Rewrite Rule Set
            * `name` (`pulumi.Input[str]`) - The name of the Application Gateway. Changing this forces a new resource to be created.
        
          * `connectionDraining` (`pulumi.Input[dict]`)
        
            * `drainTimeoutSec` (`pulumi.Input[float]`)
            * `enabled` (`pulumi.Input[bool]`)
        
          * `cookieBasedAffinity` (`pulumi.Input[str]`)
          * `host_name` (`pulumi.Input[str]`)
          * `id` (`pulumi.Input[str]`) - The ID of the Rewrite Rule Set
          * `name` (`pulumi.Input[str]`) - The name of the Application Gateway. Changing this forces a new resource to be created.
          * `path` (`pulumi.Input[str]`)
          * `pickHostNameFromBackendAddress` (`pulumi.Input[bool]`)
          * `port` (`pulumi.Input[float]`)
          * `probe_id` (`pulumi.Input[str]`) - The ID of the associated Probe.
          * `probeName` (`pulumi.Input[str]`)
          * `protocol` (`pulumi.Input[str]`)
          * `requestTimeout` (`pulumi.Input[float]`)
        
        The **custom_error_configurations** object supports the following:
        
          * `customErrorPageUrl` (`pulumi.Input[str]`)
          * `id` (`pulumi.Input[str]`) - The ID of the Rewrite Rule Set
          * `statusCode` (`pulumi.Input[str]`)
        
        The **frontend_ip_configurations** object supports the following:
        
          * `id` (`pulumi.Input[str]`) - The ID of the Rewrite Rule Set
          * `name` (`pulumi.Input[str]`) - The name of the Application Gateway. Changing this forces a new resource to be created.
          * `private_ip_address` (`pulumi.Input[str]`)
          * `privateIpAddressAllocation` (`pulumi.Input[str]`)
          * `publicIpAddressId` (`pulumi.Input[str]`)
          * `subnet_id` (`pulumi.Input[str]`)
        
        The **frontend_ports** object supports the following:
        
          * `id` (`pulumi.Input[str]`) - The ID of the Rewrite Rule Set
          * `name` (`pulumi.Input[str]`) - The name of the Application Gateway. Changing this forces a new resource to be created.
          * `port` (`pulumi.Input[float]`)
        
        The **gateway_ip_configurations** object supports the following:
        
          * `id` (`pulumi.Input[str]`) - The ID of the Rewrite Rule Set
          * `name` (`pulumi.Input[str]`) - The name of the Application Gateway. Changing this forces a new resource to be created.
          * `subnet_id` (`pulumi.Input[str]`)
        
        The **http_listeners** object supports the following:
        
          * `custom_error_configurations` (`pulumi.Input[list]`) - One or more `custom_error_configuration` blocks as defined below.
        
            * `customErrorPageUrl` (`pulumi.Input[str]`)
            * `id` (`pulumi.Input[str]`) - The ID of the Rewrite Rule Set
            * `statusCode` (`pulumi.Input[str]`)
        
          * `frontend_ip_configuration_id` (`pulumi.Input[str]`) - The ID of the associated Frontend Configuration.
          * `frontend_ip_configuration_name` (`pulumi.Input[str]`)
          * `frontendPortId` (`pulumi.Input[str]`) - The ID of the associated Frontend Port.
          * `frontendPortName` (`pulumi.Input[str]`)
          * `host_name` (`pulumi.Input[str]`)
          * `id` (`pulumi.Input[str]`) - The ID of the Rewrite Rule Set
          * `name` (`pulumi.Input[str]`) - The name of the Application Gateway. Changing this forces a new resource to be created.
          * `protocol` (`pulumi.Input[str]`)
          * `requireSni` (`pulumi.Input[bool]`)
          * `sslCertificateId` (`pulumi.Input[str]`) - The ID of the associated SSL Certificate.
          * `sslCertificateName` (`pulumi.Input[str]`)
        
        The **identity** object supports the following:
        
          * `identityIds` (`pulumi.Input[str]`)
          * `type` (`pulumi.Input[str]`)
        
        The **probes** object supports the following:
        
          * `host` (`pulumi.Input[str]`)
          * `id` (`pulumi.Input[str]`) - The ID of the Rewrite Rule Set
          * `interval` (`pulumi.Input[float]`)
          * `match` (`pulumi.Input[dict]`)
        
            * `body` (`pulumi.Input[str]`)
            * `statusCodes` (`pulumi.Input[list]`)
        
          * `minimumServers` (`pulumi.Input[float]`)
          * `name` (`pulumi.Input[str]`) - The name of the Application Gateway. Changing this forces a new resource to be created.
          * `path` (`pulumi.Input[str]`)
          * `pickHostNameFromBackendHttpSettings` (`pulumi.Input[bool]`)
          * `protocol` (`pulumi.Input[str]`)
          * `timeout` (`pulumi.Input[float]`)
          * `unhealthyThreshold` (`pulumi.Input[float]`)
        
        The **redirect_configurations** object supports the following:
        
          * `id` (`pulumi.Input[str]`) - The ID of the Rewrite Rule Set
          * `includePath` (`pulumi.Input[bool]`)
          * `includeQueryString` (`pulumi.Input[bool]`)
          * `name` (`pulumi.Input[str]`) - The name of the Application Gateway. Changing this forces a new resource to be created.
          * `redirectType` (`pulumi.Input[str]`)
          * `targetListenerId` (`pulumi.Input[str]`)
          * `targetListenerName` (`pulumi.Input[str]`)
          * `targetUrl` (`pulumi.Input[str]`)
        
        The **request_routing_rules** object supports the following:
        
          * `backend_address_pool_id` (`pulumi.Input[str]`) - The ID of the associated Backend Address Pool.
          * `backendAddressPoolName` (`pulumi.Input[str]`)
          * `backendHttpSettingsId` (`pulumi.Input[str]`) - The ID of the associated Backend HTTP Settings Configuration.
          * `backendHttpSettingsName` (`pulumi.Input[str]`)
          * `httpListenerId` (`pulumi.Input[str]`) - The ID of the associated HTTP Listener.
          * `httpListenerName` (`pulumi.Input[str]`)
          * `id` (`pulumi.Input[str]`) - The ID of the Rewrite Rule Set
          * `name` (`pulumi.Input[str]`) - The name of the Application Gateway. Changing this forces a new resource to be created.
          * `redirectConfigurationId` (`pulumi.Input[str]`) - The ID of the associated Redirect Configuration.
          * `redirectConfigurationName` (`pulumi.Input[str]`)
          * `rewriteRuleSetId` (`pulumi.Input[str]`) - The ID of the associated Rewrite Rule Set.
          * `rewriteRuleSetName` (`pulumi.Input[str]`)
          * `ruleType` (`pulumi.Input[str]`)
          * `urlPathMapId` (`pulumi.Input[str]`) - The ID of the associated URL Path Map.
          * `urlPathMapName` (`pulumi.Input[str]`)
        
        The **rewrite_rule_sets** object supports the following:
        
          * `id` (`pulumi.Input[str]`) - The ID of the Rewrite Rule Set
          * `name` (`pulumi.Input[str]`) - The name of the Application Gateway. Changing this forces a new resource to be created.
          * `rewriteRules` (`pulumi.Input[list]`)
        
            * `conditions` (`pulumi.Input[list]`)
        
              * `ignoreCase` (`pulumi.Input[bool]`)
              * `negate` (`pulumi.Input[bool]`)
              * `pattern` (`pulumi.Input[str]`)
              * `variable` (`pulumi.Input[str]`)
        
            * `name` (`pulumi.Input[str]`) - The name of the Application Gateway. Changing this forces a new resource to be created.
            * `requestHeaderConfigurations` (`pulumi.Input[list]`)
        
              * `headerName` (`pulumi.Input[str]`)
              * `headerValue` (`pulumi.Input[str]`)
        
            * `responseHeaderConfigurations` (`pulumi.Input[list]`)
        
              * `headerName` (`pulumi.Input[str]`)
              * `headerValue` (`pulumi.Input[str]`)
        
            * `ruleSequence` (`pulumi.Input[float]`)
        
        The **sku** object supports the following:
        
          * `capacity` (`pulumi.Input[float]`)
          * `name` (`pulumi.Input[str]`) - The name of the Application Gateway. Changing this forces a new resource to be created.
          * `tier` (`pulumi.Input[str]`)
        
        The **ssl_certificates** object supports the following:
        
          * `data` (`pulumi.Input[str]`)
          * `id` (`pulumi.Input[str]`) - The ID of the Rewrite Rule Set
          * `name` (`pulumi.Input[str]`) - The name of the Application Gateway. Changing this forces a new resource to be created.
          * `password` (`pulumi.Input[str]`)
          * `publicCertData` (`pulumi.Input[str]`) - The Public Certificate Data associated with the SSL Certificate.
        
        The **ssl_policies** object supports the following:
        
          * `cipherSuites` (`pulumi.Input[list]`)
          * `disabledProtocols` (`pulumi.Input[list]`)
          * `minProtocolVersion` (`pulumi.Input[str]`)
          * `policyName` (`pulumi.Input[str]`)
          * `policyType` (`pulumi.Input[str]`)
        
        The **trusted_root_certificates** object supports the following:
        
          * `data` (`pulumi.Input[str]`)
          * `id` (`pulumi.Input[str]`) - The ID of the Rewrite Rule Set
          * `name` (`pulumi.Input[str]`) - The name of the Application Gateway. Changing this forces a new resource to be created.
        
        The **url_path_maps** object supports the following:
        
          * `defaultBackendAddressPoolId` (`pulumi.Input[str]`) - The ID of the Default Backend Address Pool.
          * `defaultBackendAddressPoolName` (`pulumi.Input[str]`)
          * `defaultBackendHttpSettingsId` (`pulumi.Input[str]`) - The ID of the Default Backend HTTP Settings Collection.
          * `defaultBackendHttpSettingsName` (`pulumi.Input[str]`)
          * `defaultRedirectConfigurationId` (`pulumi.Input[str]`) - The ID of the Default Redirect Configuration.
          * `defaultRedirectConfigurationName` (`pulumi.Input[str]`)
          * `defaultRewriteRuleSetId` (`pulumi.Input[str]`)
          * `defaultRewriteRuleSetName` (`pulumi.Input[str]`)
          * `id` (`pulumi.Input[str]`) - The ID of the Rewrite Rule Set
          * `name` (`pulumi.Input[str]`) - The name of the Application Gateway. Changing this forces a new resource to be created.
          * `pathRules` (`pulumi.Input[list]`) - A list of `path_rule` blocks as defined above.
        
            * `backend_address_pool_id` (`pulumi.Input[str]`) - The ID of the associated Backend Address Pool.
            * `backendAddressPoolName` (`pulumi.Input[str]`)
            * `backendHttpSettingsId` (`pulumi.Input[str]`) - The ID of the associated Backend HTTP Settings Configuration.
            * `backendHttpSettingsName` (`pulumi.Input[str]`)
            * `id` (`pulumi.Input[str]`) - The ID of the Rewrite Rule Set
            * `name` (`pulumi.Input[str]`) - The name of the Application Gateway. Changing this forces a new resource to be created.
            * `paths` (`pulumi.Input[list]`)
            * `redirectConfigurationId` (`pulumi.Input[str]`) - The ID of the associated Redirect Configuration.
            * `redirectConfigurationName` (`pulumi.Input[str]`)
            * `rewriteRuleSetId` (`pulumi.Input[str]`) - The ID of the associated Rewrite Rule Set.
            * `rewriteRuleSetName` (`pulumi.Input[str]`)
        
        The **waf_configuration** object supports the following:
        
          * `disabledRuleGroups` (`pulumi.Input[list]`)
        
            * `ruleGroupName` (`pulumi.Input[str]`)
            * `rules` (`pulumi.Input[list]`)
        
          * `enabled` (`pulumi.Input[bool]`)
          * `exclusions` (`pulumi.Input[list]`)
        
            * `matchVariable` (`pulumi.Input[str]`)
            * `selector` (`pulumi.Input[str]`)
            * `selectorMatchOperator` (`pulumi.Input[str]`)
        
          * `fileUploadLimitMb` (`pulumi.Input[float]`)
          * `firewallMode` (`pulumi.Input[str]`)
          * `maxRequestBodySizeKb` (`pulumi.Input[float]`)
          * `requestBodyCheck` (`pulumi.Input[bool]`)
          * `ruleSetType` (`pulumi.Input[str]`)
          * `ruleSetVersion` (`pulumi.Input[str]`)

        > This content is derived from https://github.com/terraform-providers/terraform-provider-azurerm/blob/master/website/docs/r/application_gateway.html.markdown.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['authentication_certificates'] = authentication_certificates
            __props__['autoscale_configuration'] = autoscale_configuration
            if backend_address_pools is None:
                raise TypeError("Missing required property 'backend_address_pools'")
            __props__['backend_address_pools'] = backend_address_pools
            if backend_http_settings is None:
                raise TypeError("Missing required property 'backend_http_settings'")
            __props__['backend_http_settings'] = backend_http_settings
            __props__['custom_error_configurations'] = custom_error_configurations
            __props__['disabled_ssl_protocols'] = disabled_ssl_protocols
            __props__['enable_http2'] = enable_http2
            if frontend_ip_configurations is None:
                raise TypeError("Missing required property 'frontend_ip_configurations'")
            __props__['frontend_ip_configurations'] = frontend_ip_configurations
            if frontend_ports is None:
                raise TypeError("Missing required property 'frontend_ports'")
            __props__['frontend_ports'] = frontend_ports
            if gateway_ip_configurations is None:
                raise TypeError("Missing required property 'gateway_ip_configurations'")
            __props__['gateway_ip_configurations'] = gateway_ip_configurations
            if http_listeners is None:
                raise TypeError("Missing required property 'http_listeners'")
            __props__['http_listeners'] = http_listeners
            __props__['identity'] = identity
            __props__['location'] = location
            __props__['name'] = name
            __props__['probes'] = probes
            __props__['redirect_configurations'] = redirect_configurations
            if request_routing_rules is None:
                raise TypeError("Missing required property 'request_routing_rules'")
            __props__['request_routing_rules'] = request_routing_rules
            if resource_group_name is None:
                raise TypeError("Missing required property 'resource_group_name'")
            __props__['resource_group_name'] = resource_group_name
            __props__['rewrite_rule_sets'] = rewrite_rule_sets
            if sku is None:
                raise TypeError("Missing required property 'sku'")
            __props__['sku'] = sku
            __props__['ssl_certificates'] = ssl_certificates
            __props__['ssl_policies'] = ssl_policies
            __props__['tags'] = tags
            __props__['trusted_root_certificates'] = trusted_root_certificates
            __props__['url_path_maps'] = url_path_maps
            __props__['waf_configuration'] = waf_configuration
            __props__['zones'] = zones
        super(ApplicationGateway, __self__).__init__(
            'azure:network/applicationGateway:ApplicationGateway',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, authentication_certificates=None, autoscale_configuration=None, backend_address_pools=None, backend_http_settings=None, custom_error_configurations=None, disabled_ssl_protocols=None, enable_http2=None, frontend_ip_configurations=None, frontend_ports=None, gateway_ip_configurations=None, http_listeners=None, identity=None, location=None, name=None, probes=None, redirect_configurations=None, request_routing_rules=None, resource_group_name=None, rewrite_rule_sets=None, sku=None, ssl_certificates=None, ssl_policies=None, tags=None, trusted_root_certificates=None, url_path_maps=None, waf_configuration=None, zones=None):
        """
        Get an existing ApplicationGateway resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.
        
        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[list] authentication_certificates: One or more `authentication_certificate` blocks as defined below.
        :param pulumi.Input[dict] autoscale_configuration: A `autoscale_configuration` block as defined below.
        :param pulumi.Input[list] backend_address_pools: One or more `backend_address_pool` blocks as defined below.
        :param pulumi.Input[list] backend_http_settings: One or more `backend_http_settings` blocks as defined below.
        :param pulumi.Input[list] custom_error_configurations: One or more `custom_error_configuration` blocks as defined below.
        :param pulumi.Input[list] disabled_ssl_protocols: A list of SSL Protocols which should be disabled on this Application Gateway. Possible values are `TLSv1_0`, `TLSv1_1` and `TLSv1_2`.
               > **NOTE:** `disabled_ssl_protocols ` has been deprecated in favour of `disabled_protocols` in the `ssl_policy` block.
        :param pulumi.Input[bool] enable_http2: Is HTTP2 enabled on the application gateway resource? Defaults to `false`.
        :param pulumi.Input[list] frontend_ip_configurations: One or more `frontend_ip_configuration` blocks as defined below.
        :param pulumi.Input[list] frontend_ports: One or more `frontend_port` blocks as defined below.
        :param pulumi.Input[list] gateway_ip_configurations: One or more `gateway_ip_configuration` blocks as defined below.
        :param pulumi.Input[list] http_listeners: One or more `http_listener` blocks as defined below.
        :param pulumi.Input[dict] identity: A `identity` block.
        :param pulumi.Input[str] location: The Azure region where the Application Gateway should exist. Changing this forces a new resource to be created.
        :param pulumi.Input[str] name: The name of the Application Gateway. Changing this forces a new resource to be created.
        :param pulumi.Input[list] probes: One or more `probe` blocks as defined below.
        :param pulumi.Input[list] redirect_configurations: A `redirect_configuration` block as defined below.
        :param pulumi.Input[list] request_routing_rules: One or more `request_routing_rule` blocks as defined below.
        :param pulumi.Input[str] resource_group_name: The name of the resource group in which to the Application Gateway should exist. Changing this forces a new resource to be created.
        :param pulumi.Input[list] rewrite_rule_sets: One or more `rewrite_rule_set` blocks as defined below. Only valid for v2 SKUs.
        :param pulumi.Input[dict] sku: A `sku` block as defined below.
        :param pulumi.Input[list] ssl_certificates: One or more `ssl_certificate` blocks as defined below.
        :param pulumi.Input[list] ssl_policies: a `ssl policy` block as defined below.
        :param pulumi.Input[dict] tags: A mapping of tags to assign to the resource.
        :param pulumi.Input[list] trusted_root_certificates: One or more `trusted_root_certificate` blocks as defined below.
        :param pulumi.Input[list] url_path_maps: One or more `url_path_map` blocks as defined below.
        :param pulumi.Input[dict] waf_configuration: A `waf_configuration` block as defined below.
        :param pulumi.Input[list] zones: A collection of availability zones to spread the Application Gateway over.
        
        The **authentication_certificates** object supports the following:
        
          * `data` (`pulumi.Input[str]`)
          * `id` (`pulumi.Input[str]`) - The ID of the Rewrite Rule Set
          * `name` (`pulumi.Input[str]`) - The name of the Application Gateway. Changing this forces a new resource to be created.
        
        The **autoscale_configuration** object supports the following:
        
          * `maxCapacity` (`pulumi.Input[float]`)
          * `minCapacity` (`pulumi.Input[float]`)
        
        The **backend_address_pools** object supports the following:
        
          * `fqdnLists` (`pulumi.Input[list]`)
          * `fqdns` (`pulumi.Input[list]`)
          * `id` (`pulumi.Input[str]`) - The ID of the Rewrite Rule Set
          * `ipAddressLists` (`pulumi.Input[list]`)
          * `ipAddresses` (`pulumi.Input[list]`)
          * `name` (`pulumi.Input[str]`) - The name of the Application Gateway. Changing this forces a new resource to be created.
        
        The **backend_http_settings** object supports the following:
        
          * `affinityCookieName` (`pulumi.Input[str]`)
          * `authentication_certificates` (`pulumi.Input[list]`) - One or more `authentication_certificate` blocks as defined below.
        
            * `id` (`pulumi.Input[str]`) - The ID of the Rewrite Rule Set
            * `name` (`pulumi.Input[str]`) - The name of the Application Gateway. Changing this forces a new resource to be created.
        
          * `connectionDraining` (`pulumi.Input[dict]`)
        
            * `drainTimeoutSec` (`pulumi.Input[float]`)
            * `enabled` (`pulumi.Input[bool]`)
        
          * `cookieBasedAffinity` (`pulumi.Input[str]`)
          * `host_name` (`pulumi.Input[str]`)
          * `id` (`pulumi.Input[str]`) - The ID of the Rewrite Rule Set
          * `name` (`pulumi.Input[str]`) - The name of the Application Gateway. Changing this forces a new resource to be created.
          * `path` (`pulumi.Input[str]`)
          * `pickHostNameFromBackendAddress` (`pulumi.Input[bool]`)
          * `port` (`pulumi.Input[float]`)
          * `probe_id` (`pulumi.Input[str]`) - The ID of the associated Probe.
          * `probeName` (`pulumi.Input[str]`)
          * `protocol` (`pulumi.Input[str]`)
          * `requestTimeout` (`pulumi.Input[float]`)
        
        The **custom_error_configurations** object supports the following:
        
          * `customErrorPageUrl` (`pulumi.Input[str]`)
          * `id` (`pulumi.Input[str]`) - The ID of the Rewrite Rule Set
          * `statusCode` (`pulumi.Input[str]`)
        
        The **frontend_ip_configurations** object supports the following:
        
          * `id` (`pulumi.Input[str]`) - The ID of the Rewrite Rule Set
          * `name` (`pulumi.Input[str]`) - The name of the Application Gateway. Changing this forces a new resource to be created.
          * `private_ip_address` (`pulumi.Input[str]`)
          * `privateIpAddressAllocation` (`pulumi.Input[str]`)
          * `publicIpAddressId` (`pulumi.Input[str]`)
          * `subnet_id` (`pulumi.Input[str]`)
        
        The **frontend_ports** object supports the following:
        
          * `id` (`pulumi.Input[str]`) - The ID of the Rewrite Rule Set
          * `name` (`pulumi.Input[str]`) - The name of the Application Gateway. Changing this forces a new resource to be created.
          * `port` (`pulumi.Input[float]`)
        
        The **gateway_ip_configurations** object supports the following:
        
          * `id` (`pulumi.Input[str]`) - The ID of the Rewrite Rule Set
          * `name` (`pulumi.Input[str]`) - The name of the Application Gateway. Changing this forces a new resource to be created.
          * `subnet_id` (`pulumi.Input[str]`)
        
        The **http_listeners** object supports the following:
        
          * `custom_error_configurations` (`pulumi.Input[list]`) - One or more `custom_error_configuration` blocks as defined below.
        
            * `customErrorPageUrl` (`pulumi.Input[str]`)
            * `id` (`pulumi.Input[str]`) - The ID of the Rewrite Rule Set
            * `statusCode` (`pulumi.Input[str]`)
        
          * `frontend_ip_configuration_id` (`pulumi.Input[str]`) - The ID of the associated Frontend Configuration.
          * `frontend_ip_configuration_name` (`pulumi.Input[str]`)
          * `frontendPortId` (`pulumi.Input[str]`) - The ID of the associated Frontend Port.
          * `frontendPortName` (`pulumi.Input[str]`)
          * `host_name` (`pulumi.Input[str]`)
          * `id` (`pulumi.Input[str]`) - The ID of the Rewrite Rule Set
          * `name` (`pulumi.Input[str]`) - The name of the Application Gateway. Changing this forces a new resource to be created.
          * `protocol` (`pulumi.Input[str]`)
          * `requireSni` (`pulumi.Input[bool]`)
          * `sslCertificateId` (`pulumi.Input[str]`) - The ID of the associated SSL Certificate.
          * `sslCertificateName` (`pulumi.Input[str]`)
        
        The **identity** object supports the following:
        
          * `identityIds` (`pulumi.Input[str]`)
          * `type` (`pulumi.Input[str]`)
        
        The **probes** object supports the following:
        
          * `host` (`pulumi.Input[str]`)
          * `id` (`pulumi.Input[str]`) - The ID of the Rewrite Rule Set
          * `interval` (`pulumi.Input[float]`)
          * `match` (`pulumi.Input[dict]`)
        
            * `body` (`pulumi.Input[str]`)
            * `statusCodes` (`pulumi.Input[list]`)
        
          * `minimumServers` (`pulumi.Input[float]`)
          * `name` (`pulumi.Input[str]`) - The name of the Application Gateway. Changing this forces a new resource to be created.
          * `path` (`pulumi.Input[str]`)
          * `pickHostNameFromBackendHttpSettings` (`pulumi.Input[bool]`)
          * `protocol` (`pulumi.Input[str]`)
          * `timeout` (`pulumi.Input[float]`)
          * `unhealthyThreshold` (`pulumi.Input[float]`)
        
        The **redirect_configurations** object supports the following:
        
          * `id` (`pulumi.Input[str]`) - The ID of the Rewrite Rule Set
          * `includePath` (`pulumi.Input[bool]`)
          * `includeQueryString` (`pulumi.Input[bool]`)
          * `name` (`pulumi.Input[str]`) - The name of the Application Gateway. Changing this forces a new resource to be created.
          * `redirectType` (`pulumi.Input[str]`)
          * `targetListenerId` (`pulumi.Input[str]`)
          * `targetListenerName` (`pulumi.Input[str]`)
          * `targetUrl` (`pulumi.Input[str]`)
        
        The **request_routing_rules** object supports the following:
        
          * `backend_address_pool_id` (`pulumi.Input[str]`) - The ID of the associated Backend Address Pool.
          * `backendAddressPoolName` (`pulumi.Input[str]`)
          * `backendHttpSettingsId` (`pulumi.Input[str]`) - The ID of the associated Backend HTTP Settings Configuration.
          * `backendHttpSettingsName` (`pulumi.Input[str]`)
          * `httpListenerId` (`pulumi.Input[str]`) - The ID of the associated HTTP Listener.
          * `httpListenerName` (`pulumi.Input[str]`)
          * `id` (`pulumi.Input[str]`) - The ID of the Rewrite Rule Set
          * `name` (`pulumi.Input[str]`) - The name of the Application Gateway. Changing this forces a new resource to be created.
          * `redirectConfigurationId` (`pulumi.Input[str]`) - The ID of the associated Redirect Configuration.
          * `redirectConfigurationName` (`pulumi.Input[str]`)
          * `rewriteRuleSetId` (`pulumi.Input[str]`) - The ID of the associated Rewrite Rule Set.
          * `rewriteRuleSetName` (`pulumi.Input[str]`)
          * `ruleType` (`pulumi.Input[str]`)
          * `urlPathMapId` (`pulumi.Input[str]`) - The ID of the associated URL Path Map.
          * `urlPathMapName` (`pulumi.Input[str]`)
        
        The **rewrite_rule_sets** object supports the following:
        
          * `id` (`pulumi.Input[str]`) - The ID of the Rewrite Rule Set
          * `name` (`pulumi.Input[str]`) - The name of the Application Gateway. Changing this forces a new resource to be created.
          * `rewriteRules` (`pulumi.Input[list]`)
        
            * `conditions` (`pulumi.Input[list]`)
        
              * `ignoreCase` (`pulumi.Input[bool]`)
              * `negate` (`pulumi.Input[bool]`)
              * `pattern` (`pulumi.Input[str]`)
              * `variable` (`pulumi.Input[str]`)
        
            * `name` (`pulumi.Input[str]`) - The name of the Application Gateway. Changing this forces a new resource to be created.
            * `requestHeaderConfigurations` (`pulumi.Input[list]`)
        
              * `headerName` (`pulumi.Input[str]`)
              * `headerValue` (`pulumi.Input[str]`)
        
            * `responseHeaderConfigurations` (`pulumi.Input[list]`)
        
              * `headerName` (`pulumi.Input[str]`)
              * `headerValue` (`pulumi.Input[str]`)
        
            * `ruleSequence` (`pulumi.Input[float]`)
        
        The **sku** object supports the following:
        
          * `capacity` (`pulumi.Input[float]`)
          * `name` (`pulumi.Input[str]`) - The name of the Application Gateway. Changing this forces a new resource to be created.
          * `tier` (`pulumi.Input[str]`)
        
        The **ssl_certificates** object supports the following:
        
          * `data` (`pulumi.Input[str]`)
          * `id` (`pulumi.Input[str]`) - The ID of the Rewrite Rule Set
          * `name` (`pulumi.Input[str]`) - The name of the Application Gateway. Changing this forces a new resource to be created.
          * `password` (`pulumi.Input[str]`)
          * `publicCertData` (`pulumi.Input[str]`) - The Public Certificate Data associated with the SSL Certificate.
        
        The **ssl_policies** object supports the following:
        
          * `cipherSuites` (`pulumi.Input[list]`)
          * `disabledProtocols` (`pulumi.Input[list]`)
          * `minProtocolVersion` (`pulumi.Input[str]`)
          * `policyName` (`pulumi.Input[str]`)
          * `policyType` (`pulumi.Input[str]`)
        
        The **trusted_root_certificates** object supports the following:
        
          * `data` (`pulumi.Input[str]`)
          * `id` (`pulumi.Input[str]`) - The ID of the Rewrite Rule Set
          * `name` (`pulumi.Input[str]`) - The name of the Application Gateway. Changing this forces a new resource to be created.
        
        The **url_path_maps** object supports the following:
        
          * `defaultBackendAddressPoolId` (`pulumi.Input[str]`) - The ID of the Default Backend Address Pool.
          * `defaultBackendAddressPoolName` (`pulumi.Input[str]`)
          * `defaultBackendHttpSettingsId` (`pulumi.Input[str]`) - The ID of the Default Backend HTTP Settings Collection.
          * `defaultBackendHttpSettingsName` (`pulumi.Input[str]`)
          * `defaultRedirectConfigurationId` (`pulumi.Input[str]`) - The ID of the Default Redirect Configuration.
          * `defaultRedirectConfigurationName` (`pulumi.Input[str]`)
          * `defaultRewriteRuleSetId` (`pulumi.Input[str]`)
          * `defaultRewriteRuleSetName` (`pulumi.Input[str]`)
          * `id` (`pulumi.Input[str]`) - The ID of the Rewrite Rule Set
          * `name` (`pulumi.Input[str]`) - The name of the Application Gateway. Changing this forces a new resource to be created.
          * `pathRules` (`pulumi.Input[list]`) - A list of `path_rule` blocks as defined above.
        
            * `backend_address_pool_id` (`pulumi.Input[str]`) - The ID of the associated Backend Address Pool.
            * `backendAddressPoolName` (`pulumi.Input[str]`)
            * `backendHttpSettingsId` (`pulumi.Input[str]`) - The ID of the associated Backend HTTP Settings Configuration.
            * `backendHttpSettingsName` (`pulumi.Input[str]`)
            * `id` (`pulumi.Input[str]`) - The ID of the Rewrite Rule Set
            * `name` (`pulumi.Input[str]`) - The name of the Application Gateway. Changing this forces a new resource to be created.
            * `paths` (`pulumi.Input[list]`)
            * `redirectConfigurationId` (`pulumi.Input[str]`) - The ID of the associated Redirect Configuration.
            * `redirectConfigurationName` (`pulumi.Input[str]`)
            * `rewriteRuleSetId` (`pulumi.Input[str]`) - The ID of the associated Rewrite Rule Set.
            * `rewriteRuleSetName` (`pulumi.Input[str]`)
        
        The **waf_configuration** object supports the following:
        
          * `disabledRuleGroups` (`pulumi.Input[list]`)
        
            * `ruleGroupName` (`pulumi.Input[str]`)
            * `rules` (`pulumi.Input[list]`)
        
          * `enabled` (`pulumi.Input[bool]`)
          * `exclusions` (`pulumi.Input[list]`)
        
            * `matchVariable` (`pulumi.Input[str]`)
            * `selector` (`pulumi.Input[str]`)
            * `selectorMatchOperator` (`pulumi.Input[str]`)
        
          * `fileUploadLimitMb` (`pulumi.Input[float]`)
          * `firewallMode` (`pulumi.Input[str]`)
          * `maxRequestBodySizeKb` (`pulumi.Input[float]`)
          * `requestBodyCheck` (`pulumi.Input[bool]`)
          * `ruleSetType` (`pulumi.Input[str]`)
          * `ruleSetVersion` (`pulumi.Input[str]`)

        > This content is derived from https://github.com/terraform-providers/terraform-provider-azurerm/blob/master/website/docs/r/application_gateway.html.markdown.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()
        __props__["authentication_certificates"] = authentication_certificates
        __props__["autoscale_configuration"] = autoscale_configuration
        __props__["backend_address_pools"] = backend_address_pools
        __props__["backend_http_settings"] = backend_http_settings
        __props__["custom_error_configurations"] = custom_error_configurations
        __props__["disabled_ssl_protocols"] = disabled_ssl_protocols
        __props__["enable_http2"] = enable_http2
        __props__["frontend_ip_configurations"] = frontend_ip_configurations
        __props__["frontend_ports"] = frontend_ports
        __props__["gateway_ip_configurations"] = gateway_ip_configurations
        __props__["http_listeners"] = http_listeners
        __props__["identity"] = identity
        __props__["location"] = location
        __props__["name"] = name
        __props__["probes"] = probes
        __props__["redirect_configurations"] = redirect_configurations
        __props__["request_routing_rules"] = request_routing_rules
        __props__["resource_group_name"] = resource_group_name
        __props__["rewrite_rule_sets"] = rewrite_rule_sets
        __props__["sku"] = sku
        __props__["ssl_certificates"] = ssl_certificates
        __props__["ssl_policies"] = ssl_policies
        __props__["tags"] = tags
        __props__["trusted_root_certificates"] = trusted_root_certificates
        __props__["url_path_maps"] = url_path_maps
        __props__["waf_configuration"] = waf_configuration
        __props__["zones"] = zones
        return ApplicationGateway(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop

